@@ -12,9 +12,18 @@ import re
 from calibre.ebooks.metadata import MetaInformation
 from calibre.ebooks.chardet import xml_to_unicode
 
+
 def get_metadata(stream):
-    src = xml_to_unicode(stream.read())[0]
-    
+    src = stream.read()
+    return get_metadata_(src)
+
+def get_metadata_(src, encoding=None):
+    if not isinstance(src, unicode):
+        if not encoding:
+            src = xml_to_unicode(src)[0]
+        else:
+            src = src.decode(encoding, 'replace')
+
     # Title
     title = None
     pat = re.compile(r'<!--.*?TITLE=(?P<q>[\'"])(.+?)(?P=q).*?-->', re.DOTALL)
@@ -26,29 +35,29 @@ def get_metadata(stream):
         match = pat.search(src)
         if match:
             title = match.group(1)
-        
+
     # Author
     author = None
     pat = re.compile(r'<!--.*?AUTHOR=(?P<q>[\'"])(.+?)(?P=q).*?-->', re.DOTALL)
     match = pat.search(src)
     if match:
         author = match.group(2).replace(',', ';')
-        
+
     mi = MetaInformation(title, [author] if author else None)
-    
+
     # Publisher
     pat = re.compile(r'<!--.*?PUBLISHER=(?P<q>[\'"])(.+?)(?P=q).*?-->', re.DOTALL)
     match = pat.search(src)
     if match:
         mi.publisher = match.group(2)
-        
+
     # ISBN
     pat = re.compile(r'<!--.*?ISBN=[\'"]([^"\']+)[\'"].*?-->', re.DOTALL)
     match = pat.search(src)
     if match:
         isbn = match.group(1)
         mi.isbn = re.sub(r'[^0-9xX]', '', isbn)
-        
+
     return mi
-    
-    
\ No newline at end of file
+
+
