@@ -2,27 +2,26 @@ __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 '''Read/Write metadata from Open Packaging Format (.opf) files.'''
 
-import sys, re, os, glob
-import cStringIO
+import re, os
 import uuid
 from urllib import unquote, quote
 
 from calibre.constants import __appname__, __version__
-from calibre.ebooks.metadata import MetaInformation
+from calibre.ebooks.metadata import MetaInformation, string_to_authors
 from calibre.ebooks.BeautifulSoup import BeautifulStoneSoup, BeautifulSoup
 from calibre.ebooks.lrf import entity_to_unicode
-from calibre.ebooks.metadata import get_parser, Resource, ResourceCollection
+from calibre.ebooks.metadata import Resource, ResourceCollection
 from calibre.ebooks.metadata.toc import TOC
 
 class OPFSoup(BeautifulStoneSoup):
-    
+
     def __init__(self, raw):
-        BeautifulStoneSoup.__init__(self, raw,  
+        BeautifulStoneSoup.__init__(self, raw,
                                   convertEntities=BeautifulSoup.HTML_ENTITIES,
                                   selfClosingTags=['item', 'itemref', 'reference'])
 
 class ManifestItem(Resource):
-    
+
     @staticmethod
     def from_opf_manifest_item(item, basedir):
         if item.has_key('href'):
@@ -37,36 +36,36 @@ class ManifestItem(Resource):
             if mt:
                 res.mime_type = mt
             return res
-    
-    @apply
-    def media_type():
+
+    @dynamic_property
+    def media_type(self):
         def fget(self):
             return self.mime_type
         def fset(self, val):
             self.mime_type = val
         return property(fget=fget, fset=fset)
-    
-        
+
+
     def __unicode__(self):
         return u'<item id="%s" href="%s" media-type="%s" />'%(self.id, self.href(), self.media_type)
-    
+
     def __str__(self):
         return unicode(self).encode('utf-8')
-    
+
     def __repr__(self):
         return unicode(self)
-        
-    
+
+
     def __getitem__(self, index):
         if index == 0:
             return self.href()
         if index == 1:
             return self.media_type
         raise IndexError('%d out of bounds.'%index)
-        
+
 
 class Manifest(ResourceCollection):
-    
+
     @staticmethod
     def from_opf_manifest_element(manifest, dir):
         m = Manifest()
@@ -81,7 +80,7 @@ class Manifest(ResourceCollection):
             except ValueError:
                 continue
         return m
-    
+
     @staticmethod
     def from_paths(entries):
         '''
@@ -96,37 +95,37 @@ class Manifest(ResourceCollection):
             m.next_id += 1
             m.append(mi)
         return m
-    
+
     def __init__(self):
         ResourceCollection.__init__(self)
         self.next_id = 1
-            
-                
+
+
     def item(self, id):
         for i in self:
             if i.id == id:
                 return i
-            
+
     def id_for_path(self, path):
         path = os.path.normpath(os.path.abspath(path))
         for i in self:
             if i.path and os.path.normpath(i.path) == path:
-                return i.id    
-            
+                return i.id
+
     def path_for_id(self, id):
         for i in self:
             if i.id == id:
                 return i.path
 
 class Spine(ResourceCollection):
-    
+
     class Item(Resource):
-        
+
         def __init__(self, idfunc, *args, **kwargs):
             Resource.__init__(self, *args, **kwargs)
             self.is_linear = True
             self.id = idfunc(self.path)
-        
+
     @staticmethod
     def from_opf_spine_element(spine, manifest):
         s = Spine(manifest)
@@ -137,7 +136,7 @@ class Spine(ResourceCollection):
                 r.is_linear = itemref.get('linear', 'yes') == 'yes'
                 s.append(r)
         return s
-                
+
     @staticmethod
     def from_paths(paths, manifest):
         s = Spine(manifest)
@@ -147,14 +146,14 @@ class Spine(ResourceCollection):
             except:
                 continue
         return s
-            
-            
-    
+
+
+
     def __init__(self, manifest):
         ResourceCollection.__init__(self)
         self.manifest = manifest
-            
-                    
+
+
     def linear_items(self):
         for r in self:
             if r.is_linear:
@@ -164,16 +163,16 @@ class Spine(ResourceCollection):
         for r in self:
             if not r.is_linear:
                 yield r.path
-        
+
     def items(self):
         for i in self:
             yield i.path
-    
-            
+
+
 class Guide(ResourceCollection):
-    
+
     class Reference(Resource):
-        
+
         @staticmethod
         def from_opf_resource_item(ref, basedir):
             title, href, type = ref.get('title', ''), ref['href'], ref['type']
@@ -181,14 +180,14 @@ class Guide(ResourceCollection):
             res.title = title
             res.type = type
             return res
-        
+
         def __repr__(self):
             ans = '<reference type="%s" href="%s" '%(self.type, self.href())
             if self.title:
                 ans += 'title="%s" '%self.title
             return ans + '/>'
-        
-        
+
+
     @staticmethod
     def from_opf_guide(guide_elem, base_dir=os.getcwdu()):
         coll = Guide()
@@ -199,29 +198,29 @@ class Guide(ResourceCollection):
             except:
                 continue
         return coll
-        
+
     def set_cover(self, path):
         map(self.remove, [i for i in self if 'cover' in i.type.lower()])
         for type in ('cover', 'other.ms-coverimage-standard', 'other.ms-coverimage'):
             self.append(Guide.Reference(path, is_path=True))
             self[-1].type = type
             self[-1].title = ''
-        
+
 
 class standard_field(object):
-    
+
     def __init__(self, name):
         self.name = name
-        
+
     def __get__(self, obj, typ=None):
         return getattr(obj, 'get_'+self.name)()
-    
+
 
 class OPF(MetaInformation):
-    
+
     MIMETYPE = 'application/oebps-package+xml'
     ENTITY_PATTERN = re.compile(r'&(\S+?);')
-    
+
     uid            = standard_field('uid')
     application_id = standard_field('application_id')
     title          = standard_field('title')
@@ -238,29 +237,29 @@ class OPF(MetaInformation):
     series_index   = standard_field('series_index')
     rating         = standard_field('rating')
     tags           = standard_field('tags')
-    
+
     def __init__(self):
         raise NotImplementedError('Abstract base class')
-    
-    @apply
-    def package():
+
+    @dynamic_property
+    def package(self):
         def fget(self):
             return self.soup.find(re.compile('package'))
         return property(fget=fget)
-    
-    @apply
-    def metadata():
+
+    @dynamic_property
+    def metadata(self):
         def fget(self):
             return self.package.find(re.compile('metadata'))
         return property(fget=fget)
-    
-    
+
+
     def get_title(self):
         title = self.metadata.find('dc:title')
         if title and title.string:
             return self.ENTITY_PATTERN.sub(entity_to_unicode, title.string).strip()
         return self.default_title.strip()
-    
+
     def get_authors(self):
         creators = self.metadata.findAll('dc:creator')
         for elem in creators:
@@ -271,13 +270,9 @@ class OPF(MetaInformation):
                 role = 'aut'
             if role == 'aut' and elem.string:
                 raw = self.ENTITY_PATTERN.sub(entity_to_unicode, elem.string)
-                au = raw.split(',')
-                ans = []
-                for i in au:
-                    ans.extend(i.split('&'))
-                return [a.strip() for a in ans]
+                return string_to_authors(raw)
         return []
-    
+
     def get_author_sort(self):
         creators = self.metadata.findAll('dc:creator')
         for elem in creators:
@@ -288,37 +283,37 @@ class OPF(MetaInformation):
                 fa = elem.get('file-as')
                 return self.ENTITY_PATTERN.sub(entity_to_unicode, fa).strip() if fa else None
         return None
-    
+
     def get_title_sort(self):
         title = self.package.find('dc:title')
         if title:
             if title.has_key('file-as'):
                 return title['file-as'].strip()
         return None
-    
+
     def get_comments(self):
         comments = self.soup.find('dc:description')
         if comments and comments.string:
             return self.ENTITY_PATTERN.sub(entity_to_unicode, comments.string).strip()
         return None
-    
+
     def get_uid(self):
         package = self.package
         if package.has_key('unique-identifier'):
             return package['unique-identifier']
-        
+
     def get_category(self):
         category = self.soup.find('dc:type')
         if category and category.string:
             return self.ENTITY_PATTERN.sub(entity_to_unicode, category.string).strip()
         return None
-    
+
     def get_publisher(self):
         publisher = self.soup.find('dc:publisher')
         if publisher and publisher.string:
             return self.ENTITY_PATTERN.sub(entity_to_unicode, publisher.string).strip()
         return None
-    
+
     def get_isbn(self):
         for item in self.metadata.findAll('dc:identifier'):
             scheme = item.get('scheme')
@@ -327,13 +322,13 @@ class OPF(MetaInformation):
             if scheme is not None and scheme.lower() == 'isbn' and item.string:
                 return str(item.string).strip()
         return None
-    
+
     def get_language(self):
         item = self.metadata.find('dc:language')
         if not item:
             return _('Unknown')
         return ''.join(item.findAll(text=True)).strip()
-    
+
     def get_application_id(self):
         for item in self.metadata.findAll('dc:identifier'):
             scheme = item.get('scheme', None)
@@ -342,7 +337,7 @@ class OPF(MetaInformation):
             if scheme in ['libprs500', 'calibre']:
                 return str(item.string).strip()
         return None
-    
+
     def get_cover(self):
         guide = getattr(self, 'guide', [])
         if not guide:
@@ -352,7 +347,7 @@ class OPF(MetaInformation):
             matches = [r for r in references if r.type.lower() == candidate and r.path]
             if matches:
                 return matches[0].path
-    
+
     def possible_cover_prefixes(self):
         isbn, ans = [], []
         for item in self.metadata.findAll('dc:identifier'):
@@ -363,22 +358,22 @@ class OPF(MetaInformation):
         for item in isbn:
             ans.append(item[1].replace('-', ''))
         return ans
-    
+
     def get_series(self):
         s = self.metadata.find('series')
         if s is not None:
             return str(s.string).strip()
         return None
-    
+
     def get_series_index(self):
         s = self.metadata.find('series-index')
         if s and s.string:
             try:
-                return int(str(s.string).strip())
+                return float(str(s.string).strip())
             except:
                 return None
         return None
-    
+
     def get_rating(self):
         s = self.metadata.find('rating')
         if s and s.string:
@@ -387,7 +382,7 @@ class OPF(MetaInformation):
             except:
                 return None
         return None
-    
+
     def get_tags(self):
         ans = []
         subs = self.soup.findAll('dc:subject')
@@ -396,17 +391,17 @@ class OPF(MetaInformation):
             if val:
                 ans.append(val)
         return [unicode(a).strip() for a in ans]
-    
-    
+
+
 class OPFReader(OPF):
-    
+
     def __init__(self, stream, dir=os.getcwdu()):
         manage = False
         if not hasattr(stream, 'read'):
             manage = True
             dir = os.path.dirname(stream)
             stream = open(stream, 'rb')
-        self.default_title = stream.name if hasattr(stream, 'name') else 'Unknown' 
+        self.default_title = stream.name if hasattr(stream, 'name') else 'Unknown'
         if hasattr(stream, 'seek'):
             stream.seek(0)
         self.soup = OPFSoup(stream.read())
@@ -420,18 +415,18 @@ class OPFReader(OPF):
         spine = self.soup.find(re.compile('spine'))
         if spine is not None:
             self.spine = Spine.from_opf_spine_element(spine, self.manifest)
-        
+
         self.toc = TOC(base_path=dir)
         self.toc.read_from_opf(self)
         guide = self.soup.find(re.compile('guide'))
         if guide is not None:
             self.guide = Guide.from_opf_guide(guide, dir)
-        self.base_dir = dir 
+        self.base_dir = dir
         self.cover_data = (None, None)
-        
-        
+
+
 class OPFCreator(MetaInformation):
-    
+
     def __init__(self, base_path, *args, **kwargs):
         '''
         Initialize.
@@ -451,65 +446,65 @@ class OPFCreator(MetaInformation):
             self.guide = Guide()
         if self.cover:
             self.guide.set_cover(self.cover)
-        
-        
+
+
     def create_manifest(self, entries):
         '''
         Create <manifest>
-        
+
         `entries`: List of (path, mime-type) If mime-type is None it is autodetected
         '''
-        entries = map(lambda x: x if os.path.isabs(x[0]) else 
+        entries = map(lambda x: x if os.path.isabs(x[0]) else
                       (os.path.abspath(os.path.join(self.base_path, x[0])), x[1]),
                       entries)
         self.manifest = Manifest.from_paths(entries)
         self.manifest.set_basedir(self.base_path)
-        
+
     def create_manifest_from_files_in(self, files_and_dirs):
         entries = []
-        
+
         def dodir(dir):
             for spec in os.walk(dir):
                 root, files = spec[0], spec[-1]
                 for name in files:
                     path = os.path.join(root, name)
                     if os.path.isfile(path):
-                        entries.append((path, None)) 
-        
+                        entries.append((path, None))
+
         for i in files_and_dirs:
             if os.path.isdir(i):
                 dodir(i)
             else:
                 entries.append((i, None))
-                
-        self.create_manifest(entries)    
-            
+
+        self.create_manifest(entries)
+
     def create_spine(self, entries):
         '''
         Create the <spine> element. Must first call :method:`create_manifest`.
-        
+
         `entries`: List of paths
         '''
-        entries = map(lambda x: x if os.path.isabs(x) else 
+        entries = map(lambda x: x if os.path.isabs(x) else
                       os.path.abspath(os.path.join(self.base_path, x)), entries)
         self.spine = Spine.from_paths(entries, self.manifest)
-        
+
     def set_toc(self, toc):
         '''
         Set the toc. You must call :method:`create_spine` before calling this
         method.
-        
+
         :param toc: A :class:`TOC` object
         '''
         self.toc = toc
-        
+
     def create_guide(self, guide_element):
         self.guide = Guide.from_opf_guide(guide_element, self.base_path)
         self.guide.set_basedir(self.base_path)
-            
+
     def render(self, opf_stream, ncx_stream=None, ncx_manifest_entry=None):
-        from calibre.resources import opf_template
         from calibre.utils.genshi.template import MarkupTemplate
+        opf_template = open(P('templates/opf.xml'), 'rb').read()
         template = MarkupTemplate(opf_template)
         if self.manifest:
             self.manifest.set_basedir(self.base_path)
@@ -530,7 +525,7 @@ class OPFCreator(MetaInformation):
                 cover = os.path.abspath(os.path.join(self.base_path, cover))
             self.guide.set_cover(cover)
         self.guide.set_basedir(self.base_path)
-        
+
         opf = template.generate(__appname__=__appname__, mi=self, __version__=__version__).render('xml')
         if not opf.startswith('<?xml '):
             opf = '<?xml version="1.0"  encoding="UTF-8"?>\n'+opf
@@ -540,46 +535,4 @@ class OPFCreator(MetaInformation):
         if toc is not None and ncx_stream is not None:
             toc.render(ncx_stream, self.application_id)
             ncx_stream.flush()
-    
-def option_parser():
-    return get_parser('opf')
 
-def main(args=sys.argv):
-    parser = option_parser()
-    opts, args = parser.parse_args(args)
-    if len(args) != 2:
-        parser.print_help()
-        return 1
-    mi = MetaInformation(OPFReader(open(args[1], 'rb'), os.path.abspath(os.path.dirname(args[1]))))
-    write = False
-    if opts.title is not None:
-        mi.title = opts.title.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
-        write = True
-    if opts.authors is not None:
-        aus = [i.strip().replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;') for i in opts.authors.split(',')]
-        mi.authors = aus
-        write = True
-    if opts.category is not None:
-        mi.category = opts.category.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
-        write = True
-    if opts.comment is not None:
-        mi.comments = opts.comment.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
-        write = True
-    if write:
-        mo = OPFCreator(os.path.dirname(args[1]), mi)
-        ncx = cStringIO.StringIO()
-        mo.render(open(args[1], 'wb'), ncx)
-        ncx = ncx.getvalue()
-        if ncx:
-            f = glob.glob(os.path.join(os.path.dirname(args[1]), '*.ncx'))
-            if f:
-                f = open(f[0], 'wb')
-            else:
-                f = open(os.path.splitext(args[1])[0]+'.ncx', 'wb')
-            f.write(ncx)
-            f.close()
-    print MetaInformation(OPFReader(open(args[1], 'rb'), os.path.abspath(os.path.dirname(args[1]))))
-    return 0
-
-if __name__ == '__main__':
-    sys.exit(main())
