@@ -7,7 +7,7 @@ __docformat__ = 'restructuredtext en'
 lxml based OPF parser.
 '''
 
-import sys, unittest, functools, os, mimetypes, uuid, glob, cStringIO
+import re, sys, unittest, functools, os, mimetypes, uuid, glob, cStringIO
 from urllib import unquote
 from urlparse import urlparse
 
@@ -15,8 +15,7 @@ from lxml import etree
 from dateutil import parser
 
 from calibre.ebooks.chardet import xml_to_unicode
-from calibre import relpath
-from calibre.constants import __appname__, __version__
+from calibre.constants import __appname__, __version__, filesystem_encoding
 from calibre.ebooks.metadata.toc import TOC
 from calibre.ebooks.metadata import MetaInformation, string_to_authors
 
@@ -86,8 +85,8 @@ class Resource(object):
         if self.path == basedir:
             return ''+frag
         try:
-            rpath = relpath(self.path, basedir)
-        except OSError: # On windows path and basedir could be on different drives
+            rpath = os.path.relpath(self.path, basedir)
+        except ValueError: # On windows path and basedir could be on different drives
             rpath = self.path
         if isinstance(rpath, unicode):
             rpath = rpath.encode('utf-8')
@@ -169,8 +168,8 @@ class ManifestItem(Resource):
                 res.mime_type = mt
             return res
 
-    @apply
-    def media_type():
+    @dynamic_property
+    def media_type(self):
         def fget(self):
             return self.mime_type
         def fset(self, val):
@@ -258,6 +257,11 @@ class Manifest(ResourceCollection):
             if i.id == id:
                 return i.path
 
+    def type_for_id(self, id):
+        for i in self:
+            if i.id == id:
+                return i.mime_type
+
 class Spine(ResourceCollection):
 
     class Item(Resource):
@@ -382,6 +386,8 @@ class MetadataField(object):
                 ans = self.formatter(ans)
             except:
                 return None
+        if hasattr(ans, 'strip'):
+            ans = ans.strip()
         return ans
 
     def __get__(self, obj, type=None):
@@ -429,22 +435,29 @@ class OPF(object):
     spine_path      = XPath('descendant::*[re:match(name(), "spine", "i")]/*[re:match(name(), "itemref", "i")]')
     guide_path      = XPath('descendant::*[re:match(name(), "guide", "i")]/*[re:match(name(), "reference", "i")]')
 
-    title           = MetadataField('title')
+    title           = MetadataField('title', formatter=lambda x: re.sub(r'\s+', ' ', x))
     publisher       = MetadataField('publisher')
     language        = MetadataField('language')
     comments        = MetadataField('description')
-    category        = MetadataField('category')
+    category        = MetadataField('type')
+    rights          = MetadataField('rights')
     series          = MetadataField('series', is_dc=False)
-    series_index    = MetadataField('series_index', is_dc=False, formatter=int, none_is=1)
+    series_index    = MetadataField('series_index', is_dc=False, formatter=float, none_is=1)
     rating          = MetadataField('rating', is_dc=False, formatter=int)
-    timestamp       = MetadataField('date', formatter=parser.parse)
+    pubdate         = MetadataField('date', formatter=parser.parse)
+    publication_type = MetadataField('publication_type', is_dc=False)
+    timestamp       = MetadataField('timestamp', is_dc=False, formatter=parser.parse)
 
 
     def __init__(self, stream, basedir=os.getcwdu(), unquote_urls=True):
         if not hasattr(stream, 'read'):
             stream = open(stream, 'rb')
+        raw = stream.read()
+        if not raw:
+            raise ValueError('Empty file: '+getattr(stream, 'name', 'stream'))
         self.basedir  = self.base_dir = basedir
-        raw, self.encoding = xml_to_unicode(stream.read(), strip_encoding_pats=True, resolve_entities=True)
+        self.path_to_html_toc = self.html_toc_fragment = None
+        raw, self.encoding = xml_to_unicode(raw, strip_encoding_pats=True, resolve_entities=True)
         raw = raw[raw.find('<'):]
         self.root     = etree.fromstring(raw, self.PARSER)
         self.metadata = self.metadata_path(self.root)
@@ -486,7 +499,10 @@ class OPF(object):
 
             if toc is None: return
             self.toc = TOC(base_path=self.base_dir)
-            if toc.lower() in ('ncx', 'ncxtoc'):
+            is_ncx = getattr(self, 'manifest', None) is not None and \
+                     self.manifest.type_for_id(toc) is not None and \
+                     'dtbncx' in self.manifest.type_for_id(toc)
+            if is_ncx or toc.lower() in ('ncx', 'ncxtoc'):
                 path = self.manifest.path_for_id(toc)
                 if path:
                     self.toc.read_ncx_toc(path)
@@ -495,12 +511,15 @@ class OPF(object):
                     if f:
                         self.toc.read_ncx_toc(f[0])
             else:
+                self.path_to_html_toc, self.html_toc_fragment = \
+                    toc.partition('#')[0], toc.partition('#')[-1]
+                if not os.access(self.path_to_html_toc, os.R_OK) or \
+                        not os.path.isfile(self.path_to_html_toc):
+                    self.path_to_html_toc = None
                 self.toc.read_html_toc(toc)
         except:
             pass
 
-
-
     def get_text(self, elem):
         return u''.join(self.CONTENT(elem) or self.TEXT(elem))
 
@@ -545,7 +564,7 @@ class OPF(object):
                 has_path = True
                 break
         if not has_path:
-            href = relpath(path, self.base_dir).replace(os.sep, '/')
+            href = os.path.relpath(path, self.base_dir).replace(os.sep, '/')
             item = self.create_manifest_item(href, media_type)
             manifest = self.manifest_ppath(self.root)[0]
             manifest.append(item)
@@ -608,8 +627,8 @@ class OPF(object):
         for item in self.iterguide():
             item.set('href', get_href(item))
 
-    @apply
-    def authors():
+    @dynamic_property
+    def authors(self):
 
         def fget(self):
             ans = []
@@ -625,11 +644,11 @@ class OPF(object):
                 attrib = {'{%s}role'%self.NAMESPACES['opf']: 'aut'}
                 elem = self.create_metadata_element('creator', attrib=attrib)
                 self.set_text(elem, author.strip())
-            
+
         return property(fget=fget, fset=fset)
 
-    @apply
-    def author_sort():
+    @dynamic_property
+    def author_sort(self):
 
         def fget(self):
             matches = self.authors_path(self.metadata)
@@ -651,8 +670,8 @@ class OPF(object):
 
         return property(fget=fget, fset=fset)
 
-    @apply
-    def title_sort():
+    @dynamic_property
+    def title_sort(self):
 
         def fget(self):
             matches = self.title_path(self.metadata)
@@ -674,8 +693,8 @@ class OPF(object):
 
         return property(fget=fget, fset=fset)
 
-    @apply
-    def tags():
+    @dynamic_property
+    def tags(self):
 
         def fget(self):
             ans = []
@@ -692,8 +711,8 @@ class OPF(object):
 
         return property(fget=fget, fset=fset)
 
-    @apply
-    def isbn():
+    @dynamic_property
+    def isbn(self):
 
         def fget(self):
             for match in self.isbn_path(self.metadata):
@@ -709,8 +728,8 @@ class OPF(object):
 
         return property(fget=fget, fset=fset)
 
-    @apply
-    def application_id():
+    @dynamic_property
+    def application_id(self):
 
         def fget(self):
             for match in self.application_id_path(self.metadata):
@@ -726,8 +745,8 @@ class OPF(object):
 
         return property(fget=fget, fset=fset)
 
-    @apply
-    def book_producer():
+    @dynamic_property
+    def book_producer(self):
 
         def fget(self):
             for match in self.bkp_path(self.metadata):
@@ -764,8 +783,8 @@ class OPF(object):
                             return cpath
 
 
-    @apply
-    def cover():
+    @dynamic_property
+    def cover(self):
 
         def fget(self):
             if self.guide is not None:
@@ -910,13 +929,16 @@ class OPFCreator(MetaInformation):
         self.guide.set_basedir(self.base_path)
 
     def render(self, opf_stream=sys.stdout, ncx_stream=None,
-               ncx_manifest_entry=None):
-        from calibre.resources import opf_template
+               ncx_manifest_entry=None, encoding=None):
         from calibre.utils.genshi.template import MarkupTemplate
+        opf_template = open(P('templates/opf.xml'), 'rb').read()
+        if encoding is None:
+            encoding = 'utf-8'
         template = MarkupTemplate(opf_template)
+        toc = getattr(self, 'toc', None)
         if self.manifest:
             self.manifest.set_basedir(self.base_path)
-            if ncx_manifest_entry is not None:
+            if ncx_manifest_entry is not None and toc is not None:
                 if not os.path.isabs(ncx_manifest_entry):
                     ncx_manifest_entry = os.path.join(self.base_path, ncx_manifest_entry)
                 remove = [i for i in self.manifest if i.id == 'ncx']
@@ -933,15 +955,144 @@ class OPFCreator(MetaInformation):
                 cover = os.path.abspath(os.path.join(self.base_path, cover))
             self.guide.set_cover(cover)
         self.guide.set_basedir(self.base_path)
-        opf = template.generate(__appname__=__appname__, mi=self, __version__=__version__).render('xml')
+        opf = template.generate(
+                __appname__=__appname__, mi=self,
+                __version__=__version__).render('xml', encoding=encoding)
+        opf_stream.write('<?xml version="1.0" encoding="%s" ?>\n'
+                %encoding.upper())
         opf_stream.write(opf)
         opf_stream.flush()
-        toc = getattr(self, 'toc', None)
         if toc is not None and ncx_stream is not None:
             toc.render(ncx_stream, self.application_id)
             ncx_stream.flush()
 
 
+def metadata_to_opf(mi, as_string=True):
+    from lxml import etree
+    import textwrap
+    from calibre.ebooks.oeb.base import OPF, DC
+
+    if not mi.application_id:
+        mi.application_id = str(uuid.uuid4())
+
+    if not mi.book_producer:
+        mi.book_producer = __appname__ + ' (%s) '%__version__ + \
+            '[http://calibre-ebook.com]'
+
+    if not mi.language:
+        mi.language = 'UND'
+
+    root = etree.fromstring(textwrap.dedent(
+    '''
+    <package xmlns="http://www.idpf.org/2007/opf" unique-identifier="%(a)s_id">
+        <metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
+            <dc:identifier opf:scheme="%(a)s" id="%(a)s_id">%(id)s</dc:identifier>
+        </metadata>
+        <guide/>
+    </package>
+    '''%dict(a=__appname__, id=mi.application_id)))
+    metadata = root[0]
+    guide = root[1]
+    metadata[0].tail = '\n'+(' '*8)
+    def factory(tag, text=None, sort=None, role=None, scheme=None, name=None,
+            content=None):
+        attrib = {}
+        if sort:
+            attrib[OPF('file-as')] = sort
+        if role:
+            attrib[OPF('role')] = role
+        if scheme:
+            attrib[OPF('scheme')] = scheme
+        if name:
+            attrib['name'] = name
+        if content:
+            attrib['content'] = content
+        elem = metadata.makeelement(tag, attrib=attrib)
+        elem.tail = '\n'+(' '*8)
+        if text:
+            elem.text = text.strip()
+        metadata.append(elem)
+
+    factory(DC('title'), mi.title, mi.title_sort)
+    for au in mi.authors:
+        factory(DC('creator'), au, mi.author_sort, 'aut')
+    factory(DC('contributor'), mi.book_producer, __appname__, 'bkp')
+    if hasattr(mi.pubdate, 'isoformat'):
+        factory(DC('date'), mi.pubdate.isoformat())
+    factory(DC('language'), mi.language)
+    if mi.category:
+        factory(DC('type'), mi.category)
+    if mi.comments:
+        factory(DC('description'), mi.comments)
+    if mi.publisher:
+        factory(DC('publisher'), mi.publisher)
+    if mi.isbn:
+        factory(DC('identifier'), mi.isbn, scheme='ISBN')
+    if mi.rights:
+        factory(DC('rights'), mi.rights)
+    if mi.tags:
+        for tag in mi.tags:
+            factory(DC('subject'), tag)
+    meta = lambda n, c: factory('meta', name='calibre:'+n, content=c)
+    if mi.series:
+        meta('series', mi.series)
+    if mi.series_index is not None:
+        meta('series_index', mi.format_series_index())
+    if mi.rating is not None:
+        meta('rating', str(mi.rating))
+    if hasattr(mi.timestamp, 'isoformat'):
+        meta('timestamp', mi.timestamp.isoformat())
+    if mi.publication_type:
+        meta('publication_type', mi.publication_type)
+
+    metadata[-1].tail = '\n' +(' '*4)
+
+    if mi.cover:
+        if not isinstance(mi.cover, unicode):
+            mi.cover = mi.cover.decode(filesystem_encoding)
+        guide.text = '\n'+(' '*8)
+        r = guide.makeelement(OPF('reference'),
+                attrib={'type':'cover', 'title':_('Cover'), 'href':mi.cover})
+        r.tail = '\n' +(' '*4)
+        guide.append(r)
+    return etree.tostring(root, pretty_print=True, encoding='utf-8',
+            xml_declaration=True) if as_string else root
+
+
+def test_m2o():
+    from datetime import datetime
+    from cStringIO import StringIO
+    mi = MetaInformation('test & title', ['a"1', "a'2"])
+    mi.title_sort = 'a\'"b'
+    mi.author_sort = 'author sort'
+    mi.pubdate = datetime.now()
+    mi.language = 'en'
+    mi.category = 'test'
+    mi.comments = 'what a fun book\n\n'
+    mi.publisher = 'publisher'
+    mi.isbn = 'boooo'
+    mi.tags = ['a', 'b']
+    mi.series = 's"c\'l&<>'
+    mi.series_index = 3.34
+    mi.rating = 3
+    mi.timestamp = datetime.now()
+    mi.publication_type = 'ooooo'
+    mi.rights = 'yes'
+    mi.cover = 'asd.jpg'
+    opf = metadata_to_opf(mi)
+    print opf
+    newmi = MetaInformation(OPF(StringIO(opf)))
+    for attr in ('author_sort', 'title_sort', 'comments', 'category',
+                    'publisher', 'series', 'series_index', 'rating',
+                    'isbn', 'tags', 'cover_data', 'application_id',
+                    'language', 'cover',
+                    'book_producer', 'timestamp', 'lccn', 'lcc', 'ddc',
+                    'pubdate', 'rights', 'publication_type'):
+        o, n = getattr(mi, attr), getattr(newmi, attr)
+        if o != n and o.strip() != n.strip():
+            print 'FAILED:', attr, getattr(mi, attr), '!=', getattr(newmi, attr)
+
+
 class OPFTest(unittest.TestCase):
 
     def setUp(self):
@@ -1006,57 +1157,3 @@ def suite():
 
 def test():
     unittest.TextTestRunner(verbosity=2).run(suite())
-
-
-def option_parser():
-    from calibre.ebooks.metadata import get_parser
-    parser = get_parser('opf')
-    parser.add_option('--language', default=None, help=_('Set the dc:language field'))
-    return parser
-
-def main(args=sys.argv):
-    parser = option_parser()
-    opts, args = parser.parse_args(args)
-    if len(args) != 2:
-        parser.print_help()
-        return 1
-    opfpath = os.path.abspath(args[1])
-    basedir = os.path.dirname(opfpath)
-    mi = MetaInformation(OPF(open(opfpath, 'rb'), basedir))
-    write = False
-    if opts.title is not None:
-        mi.title = opts.title
-        write = True
-    if opts.authors is not None:
-        aus = [i.strip() for i in opts.authors.split(',')]
-        mi.authors = aus
-        write = True
-    if opts.category is not None:
-        mi.category = opts.category
-        write = True
-    if opts.comment is not None:
-        mi.comments = opts.comment
-        write = True
-    if opts.language is not None:
-        mi.language = opts.language
-        write = True
-    if write:
-        mo = OPFCreator(basedir, mi)
-        ncx = cStringIO.StringIO()
-        mo.render(open(args[1], 'wb'), ncx)
-        ncx = ncx.getvalue()
-        if ncx:
-            f = glob.glob(os.path.join(os.path.dirname(args[1]), '*.ncx'))
-            if f:
-                f = open(f[0], 'wb')
-            else:
-                f = open(os.path.splitext(args[1])[0]+'.ncx', 'wb')
-            f.write(ncx)
-            f.close()
-    print MetaInformation(OPF(open(opfpath, 'rb'), basedir))
-    return 0
-
-
-
-if __name__ == '__main__':
-    sys.exit(main())
