@@ -11,20 +11,21 @@ __copyright__ = '2008, Marshall T. Vandegrift <llasram@gmail.com>'
 import os
 import itertools
 import re
+import logging
 import copy
 from weakref import WeakKeyDictionary
 from xml.dom import SyntaxErr as CSSSyntaxError
 import cssutils
 from cssutils.css import CSSStyleRule, CSSPageRule, CSSStyleDeclaration, \
-    CSSValueList, cssproperties
-from cssutils.profiles import profiles as cssprofiles
+    CSSValueList, CSSFontFaceRule, cssproperties
+from cssutils import profile as cssprofiles
 from lxml import etree
-from lxml.cssselect import css_to_xpath, ExpressionError
+from lxml.cssselect import css_to_xpath, ExpressionError, SelectorSyntaxError
 from calibre.ebooks.oeb.base import XHTML, XHTML_NS, CSS_MIME, OEB_STYLES
 from calibre.ebooks.oeb.base import XPNSMAP, xpath, urlnormalize
 from calibre.ebooks.oeb.profile import PROFILES
-from calibre.resources import html_css
 
+html_css = open(P('templates/html.css'), 'rb').read()
 XHTML_CSS_NAMESPACE = '@namespace "%s";\n' % XHTML_NS
 HTML_CSS_STYLESHEET = cssutils.parseString(html_css)
 HTML_CSS_STYLESHEET.namespaces['h'] = XHTML_NS
@@ -88,10 +89,13 @@ FONT_SIZE_NAMES = set(['xx-small', 'x-small', 'small', 'medium', 'large',
 class CSSSelector(etree.XPath):
     MIN_SPACE_RE = re.compile(r' *([>~+]) *')
     LOCAL_NAME_RE = re.compile(r"(?<!local-)name[(][)] *= *'[^:]+:")
-    
+
     def __init__(self, css, namespaces=XPNSMAP):
         css = self.MIN_SPACE_RE.sub(r'\1', css)
-        path = css_to_xpath(css)
+        try:
+            path = css_to_xpath(css)
+        except UnicodeEncodeError: # Bug in css_to_xpath
+            path = '/'
         path = self.LOCAL_NAME_RE.sub(r"local-name() = '", path)
         etree.XPath.__init__(self, path, namespaces=namespaces)
         self.css = css
@@ -103,10 +107,11 @@ class CSSSelector(etree.XPath):
             self.css)
 
 
-class Stylizer(object):    
+class Stylizer(object):
     STYLESHEETS = WeakKeyDictionary()
-    
-    def __init__(self, tree, path, oeb, profile=PROFILES['PRS505']):
+
+    def __init__(self, tree, path, oeb, profile=PROFILES['PRS505'],
+            extra_css='', user_css=''):
         self.oeb = oeb
         self.profile = profile
         self.logger = oeb.logger
@@ -115,18 +120,20 @@ class Stylizer(object):
         cssname = os.path.splitext(basename)[0] + '.css'
         stylesheets = [HTML_CSS_STYLESHEET]
         head = xpath(tree, '/h:html/h:head')[0]
-        parser = cssutils.CSSParser()
-        parser.setFetcher(self._fetch_css_file)
+        parser = cssutils.CSSParser(fetcher=self._fetch_css_file,
+                log=logging.getLogger('calibre.css'))
+        self.font_face_rules = []
         for elem in head:
             if elem.tag == XHTML('style') and elem.text \
                and elem.get('type', CSS_MIME) in OEB_STYLES:
                 text = XHTML_CSS_NAMESPACE + elem.text
+                text = oeb.css_preprocessor(text)
                 stylesheet = parser.parseString(text, href=cssname)
                 stylesheet.namespaces['h'] = XHTML_NS
                 stylesheets.append(stylesheet)
             elif elem.tag == XHTML('link') and elem.get('href') \
-                 and elem.get('rel', 'stylesheet') == 'stylesheet' \
-                 and elem.get('type', CSS_MIME) in OEB_STYLES:
+                 and elem.get('rel', 'stylesheet').lower() == 'stylesheet' \
+                 and elem.get('type', CSS_MIME).lower() in OEB_STYLES:
                 href = urlnormalize(elem.attrib['href'])
                 path = item.abshref(href)
                 sitem = oeb.manifest.hrefs.get(path, None)
@@ -135,13 +142,17 @@ class Stylizer(object):
                         'Stylesheet %r referenced by file %r not in manifest' %
                         (path, item.href))
                     continue
-                if sitem in self.STYLESHEETS:
-                    stylesheet = self.STYLESHEETS[sitem]
-                else:
-                    data = self._fetch_css_file(path)[1]
-                    stylesheet = parser.parseString(data, href=path)
-                    stylesheet.namespaces['h'] = XHTML_NS
-                    self.STYLESHEETS[sitem] = stylesheet
+                if not hasattr(sitem.data, 'cssRules'):
+                    self.logger.warn(
+                    'Stylesheet %r referenced by file %r is not CSS'%(path,
+                        item.href))
+                    continue
+                stylesheets.append(sitem.data)
+        for x in (extra_css, user_css):
+            if x:
+                text = XHTML_CSS_NAMESPACE + x
+                stylesheet = parser.parseString(text, href=cssname)
+                stylesheet.namespaces['h'] = XHTML_NS
                 stylesheets.append(stylesheet)
         rules = []
         index = 0
@@ -156,24 +167,42 @@ class Stylizer(object):
         rules.sort()
         self.rules = rules
         self._styles = {}
+        class_sel_pat = re.compile(r'\.[a-z]+', re.IGNORECASE)
         for _, _, cssdict, text, _ in rules:
             try:
                 selector = CSSSelector(text)
-            except ExpressionError:
+            except (AssertionError, ExpressionError, etree.XPathSyntaxError,
+                    NameError, # thrown on OS X instead of SelectorSyntaxError
+                    SelectorSyntaxError):
                 continue
-            for elem in selector(tree):
+            matches = selector(tree)
+            if not matches and class_sel_pat.match(text):
+                found = False
+                for x in tree.xpath('//*[@class]'):
+                    if text.lower().endswith('.'+x.get('class').lower()) and \
+                            text.lower() != text:
+                        matches.append(x)
+                        found = True
+                if found:
+                    self.logger.warn('Ignoring case mismatches for CSS selector: %s in %s'
+                        %(text, item.href))
+            for elem in matches:
                 self.style(elem)._update_cssdict(cssdict)
         for elem in xpath(tree, '//h:*[@style]'):
             self.style(elem)._apply_style_attr()
-    
+
     def _fetch_css_file(self, path):
         hrefs = self.oeb.manifest.hrefs
         if path not in hrefs:
+            self.logger.warn('CSS import of missing file %r' % path)
             return (None, None)
-        data = hrefs[path].data
-        data = XHTML_CSS_NAMESPACE + data
+        item = hrefs[path]
+        if item.media_type not in OEB_STYLES:
+            self.logger.warn('CSS import of non-CSS file %r' % path)
+            return (None, None)
+        data = item.data.cssText
         return ('utf-8', data)
-    
+
     def flatten_rule(self, rule, href, index):
         results = []
         if isinstance(rule, CSSStyleRule):
@@ -186,8 +215,10 @@ class Stylizer(object):
         elif isinstance(rule, CSSPageRule):
             style = self.flatten_style(rule.style)
             self.page_rule.update(style)
+        elif isinstance(rule, CSSFontFaceRule):
+            self.font_face_rules.append(rule)
         return results
-    
+
     def flatten_style(self, cssstyle):
         style = {}
         for prop in cssstyle:
@@ -201,10 +232,11 @@ class Stylizer(object):
         if 'font-size' in style:
             size = style['font-size']
             if size == 'normal': size = 'medium'
+            if size == 'smallest': size = 'xx-small'
             if size in FONT_SIZE_NAMES:
                 style['font-size'] = "%dpt" % self.profile.fnames[size]
         return style
-    
+
     def _normalize_edge(self, cssvalue, name):
         style = {}
         if isinstance(cssvalue, CSSValueList):
@@ -226,7 +258,7 @@ class Stylizer(object):
         for edge, value in itertools.izip(edges, values):
             style["%s-%s" % (name, edge)] = value
         return style
-        
+
     def _normalize_font(self, cssvalue):
         composition = ('font-style', 'font-variant', 'font-weight',
                        'font-size', 'line-height', 'font-family')
@@ -272,8 +304,8 @@ class Stylizer(object):
 
 
 class Style(object):
-    UNIT_RE = re.compile(r'^(-*[0-9]*[.]?[0-9]*)\s*(%|em|px|mm|cm|in|pt|pc)$')
-    
+    UNIT_RE = re.compile(r'^(-*[0-9]*[.]?[0-9]*)\s*(%|em|ex|en|px|mm|cm|in|pt|pc)$')
+
     def __init__(self, element, stylizer):
         self._element = element
         self._profile = stylizer.profile
@@ -285,9 +317,12 @@ class Style(object):
         self._lineHeight = None
         stylizer._styles[element] = self
 
+    def set(self, prop, val):
+        self._style[prop] = val
+
     def _update_cssdict(self, cssdict):
         self._style.update(cssdict)
-    
+
     def _apply_style_attr(self):
         attrib = self._element.attrib
         if 'style' not in attrib:
@@ -299,7 +334,7 @@ class Style(object):
         except CSSSyntaxError:
             return
         self._style.update(self._stylizer.flatten_style(style))
-    
+
     def _has_parent(self):
         return (self._element.getparent() is not None)
 
@@ -348,10 +383,15 @@ class Style(object):
             elif unit == 'in':
                 result = value * 72.0
             elif unit == 'pt':
-                result = value 
+                result = value
             elif unit == 'em':
                 font = font or self.fontSize
                 result = value * font
+            elif unit in ('ex', 'en'):
+                # This is a hack for ex since we have no way to know
+                # the x-height of the font
+                font = font or self.fontSize
+                result = value * font * 0.5
             elif unit == 'pc':
                 result = value * 12.0
             elif unit == 'mm':
@@ -363,6 +403,7 @@ class Style(object):
     @property
     def fontSize(self):
         def normalize_fontsize(value, base):
+            value = value.replace('"', '').replace("'", '')
             result = None
             factor = None
             if value == 'inherit':
@@ -383,6 +424,8 @@ class Style(object):
                     result = size
             else:
                 result = self._unit_convert(value, base=base, font=base)
+                if not isinstance(result, (int, float, long)):
+                    return base
                 if result < 0:
                     result = normalize_fontsize("smaller", base)
             if factor:
@@ -423,7 +466,7 @@ class Style(object):
                 result = self._unit_convert(width, base=base)
             self._width = result
         return self._width
-    
+
     @property
     def height(self):
         if self._height is None:
@@ -465,27 +508,27 @@ class Style(object):
                 result = 1.2 * self.fontSize
             self._lineHeight = result
         return self._lineHeight
-    
+
     @property
     def marginTop(self):
         return self._unit_convert(
             self._get('margin-top'), base=self.height)
-    
+
     @property
     def marginBottom(self):
         return self._unit_convert(
             self._get('margin-bottom'), base=self.height)
-    
+
     @property
     def paddingTop(self):
         return self._unit_convert(
             self._get('padding-top'), base=self.height)
-    
+
     @property
     def paddingBottom(self):
         return self._unit_convert(
             self._get('padding-bottom'), base=self.height)
-    
+
     def __str__(self):
         items = self._style.items()
         items.sort()
