@@ -9,28 +9,31 @@ Scheduler for automated recipe downloads
 
 import sys, copy, time
 from datetime import datetime, timedelta, date
-from PyQt4.Qt import QDialog, QApplication, QLineEdit, QPalette, SIGNAL, QBrush, \
+from PyQt4.Qt import QDialog, QApplication, SIGNAL, \
                      QColor, QAbstractItemModel, Qt, QVariant, QFont, QIcon, \
                      QFile, QObject, QTimer, QMutex, QMenu, QAction, QTime, QModelIndex
 
 from calibre import english_sort
 from calibre.gui2.dialogs.scheduler_ui import Ui_Dialog
+from calibre.gui2.search_box import SearchBox2
 from calibre.web.feeds.recipes import recipes, recipe_modules, compile_recipe
 from calibre.utils.search_query_parser import SearchQueryParser
 from calibre.utils.pyparsing import ParseException
+from calibre.utils.localization import get_language
 from calibre.gui2 import NONE, error_dialog, config as gconf
 from calibre.utils.config import DynamicConfig
+from calibre.ptempfile import PersistentTemporaryFile
 from calibre.gui2.dialogs.user_profiles import UserProfiles
 
 config = DynamicConfig('scheduler')
 
 class Recipe(object):
-    
+
     def __init__(self, id=None, recipe_class=None, builtin=True):
         self.id                 = id
         self.title              = getattr(recipe_class, 'title', None)
         self.description        = getattr(recipe_class, 'description', None)
-        self.language           = getattr(recipe_class, 'language', _('Unknown'))
+        self.language           = getattr(recipe_class, 'language', 'und')
         self.last_downloaded    = datetime.fromordinal(1)
         self.downloading        = False
         self.builtin            = builtin
@@ -39,14 +42,14 @@ class Recipe(object):
         if self.author == _('Unknown') and not builtin:
             self.author = _('You')
         self.needs_subscription = getattr(recipe_class, 'needs_subscription', False)
-        
+
     def pickle(self):
         return self.__dict__.copy()
-    
+
     def unpickle(self, dict):
         self.__dict__.update(dict)
         return self
-        
+
     def __cmp__(self, other):
         if self.id == getattr(other, 'id', None):
             return 0
@@ -59,43 +62,44 @@ class Recipe(object):
         if not self.builtin and getattr(other, 'builtin', True):
             return -1
         return english_sort(self.title, getattr(other, 'title', ''))
-    
+
     def __hash__(self):
         return hash(self.id)
-    
+
     def __eq__(self, other):
         return self.id == getattr(other, 'id', None)
-    
+
     def __repr__(self):
         schedule = self.schedule
         if schedule and schedule > 1e5:
             schedule = decode_schedule(schedule)
         return u'%s|%s|%s|%s'%(self.id, self.title, self.last_downloaded.ctime(), schedule)
-    
+
 builtin_recipes = [Recipe(m, r, True) for r, m in zip(recipes, recipe_modules)]
 
 def save_recipes(recipes):
     config['scheduled_recipes'] = [r.pickle() for r in recipes]
-    
+
 def load_recipes():
     config.refresh()
     recipes = []
     for r in config.get('scheduled_recipes', []):
         r = Recipe().unpickle(r)
-        if r.builtin and not str(r.id).startswith('recipe_'):
+        if r.builtin and \
+            (not str(r.id).startswith('recipe_') or not str(r.id) in recipe_modules):
             continue
         recipes.append(r)
     return recipes
 
 class RecipeModel(QAbstractItemModel, SearchQueryParser):
-    
+
     LOCATIONS = ['all']
-    
+
     def __init__(self, db, *args):
         QAbstractItemModel.__init__(self, *args)
         SearchQueryParser.__init__(self)
-        self.default_icon = QIcon(':/images/news.svg')
-        self.custom_icon = QIcon(':/images/user_profile.svg')
+        self.default_icon = QIcon(I('news.svg'))
+        self.custom_icon = QIcon(I('user_profile.svg'))
         self.recipes = copy.deepcopy(builtin_recipes)
         for x in db.get_recipes():
             recipe = compile_recipe(x[1])
@@ -104,21 +108,21 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
         self.bold_font = QFont()
         self.bold_font.setBold(True)
         self.bold_font = QVariant(self.bold_font)
-        
-    
+
+
     def refresh(self):
         sr = load_recipes()
         for recipe in self.recipes:
             if recipe in sr:
                 recipe.schedule = sr[sr.index(recipe)].schedule
                 recipe.last_downloaded = sr[sr.index(recipe)].last_downloaded
-        
+
         self.recipes.sort()
         self.num_of_recipes = len(self.recipes)
-        
+
         self.category_map = {}
         for r in self.recipes:
-            category = getattr(r, 'language', _('Unknown'))
+            category = get_language(getattr(r, 'language', 'und'))
             if not r.builtin:
                 category = _('Custom')
             if r.schedule is not None:
@@ -126,12 +130,16 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
             if category not in self.category_map.keys():
                 self.category_map[category] = []
             self.category_map[category].append(r)
-            
+
         self.categories = sorted(self.category_map.keys(), cmp=self.sort_categories)
         self._map = dict(self.category_map)
-        
+
+    def scheduled_recipes(self):
+        for recipe in self.category_map.get(_('Scheduled'), []):
+            yield recipe
+
     def sort_categories(self, x, y):
-        
+
         def decorate(x):
             if x == _('Scheduled'):
                 x = '0' + x
@@ -140,13 +148,13 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
             else:
                 x = '2' + x
             return x
-        
+
         return cmp(decorate(x), decorate(y))
-                
-    
+
+
     def universal_set(self):
         return set(self.recipes)
-    
+
     def get_matches(self, location, query):
         query = query.strip().lower()
         if not query:
@@ -154,10 +162,10 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
         results = set([])
         for recipe in self.recipes:
             if query in recipe.title.lower() or query in recipe.description.lower():
-                results.add(recipe) 
+                results.add(recipe)
         return results
-    
-    def search(self, query):
+
+    def search(self, query, refinement):
         try:
             results = self.parse(unicode(query))
         except ParseException:
@@ -170,24 +178,25 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
                     if recipe in results:
                         self._map[category].append(recipe)
         self.reset()
-    
+        self.emit(SIGNAL('searched(PyQt_PyObject)'), True)
+
     def resort(self):
         self.recipes.sort()
         self.reset()
-    
+
     def index(self, row, column, parent):
-        return self.createIndex(row, column, parent.row() if parent.isValid() else -1)
-    
+        return self.createIndex(row, column, parent.row()+1 if parent.isValid() else 0)
+
     def parent(self, index):
-        if index.internalId() == -1:
+        if index.internalId() == 0:
             return QModelIndex()
-        return self.createIndex(index.internalId(), 0, -1)
-    
+        return self.createIndex(index.internalId()-1, 0, 0)
+
     def columnCount(self, parent):
         if not parent.isValid() or not parent.parent().isValid():
             return 1
         return 0
-    
+
     def rowCount(self, parent):
         if not parent.isValid():
             return len(self.categories)
@@ -195,7 +204,7 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
             category = self.categories[parent.row()]
             return len(self._map[category])
         return 0
-        
+
     def data(self, index, role):
         if index.parent().isValid():
             category = self.categories[index.parent().row()]
@@ -206,7 +215,7 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
                 return recipe
             elif role == Qt.DecorationRole:
                 icon = self.default_icon
-                icon_path = (':/images/news/%s.png'%recipe.id).replace('recipe_', '') 
+                icon_path = (I('news/%s.png')%recipe.id).replace('recipe_', '')
                 if not recipe.builtin:
                     icon = self.custom_icon
                 elif QFile().exists(icon_path):
@@ -219,52 +228,15 @@ class RecipeModel(QAbstractItemModel, SearchQueryParser):
                 return QVariant(category + ' [%d]'%num)
             elif role == Qt.FontRole:
                 return self.bold_font
+            elif role == Qt.ForegroundRole and category == _('Scheduled'):
+                return QVariant(QColor(0, 255, 0))
         return NONE
-    
+
     def update_recipe_schedule(self, recipe):
         for srecipe in self.recipes:
             if srecipe == recipe:
                 srecipe.schedule = recipe.schedule
-            
 
-class Search(QLineEdit):
-    
-    HELP_TEXT = _('Search')
-    INTERVAL = 500 #: Time to wait before emitting search signal
-    
-    def __init__(self, *args):
-        QLineEdit.__init__(self, *args)
-        self.default_palette = QApplication.palette(self)
-        self.gray = QPalette(self.default_palette)
-        self.gray.setBrush(QPalette.Text, QBrush(QColor('gray')))
-        self.connect(self, SIGNAL('editingFinished()'),
-                     lambda : self.emit(SIGNAL('goto(PyQt_PyObject)'), unicode(self.text())))
-        self.clear_to_help_mode()
-        self.timer = None
-        self.connect(self, SIGNAL('textEdited(QString)'), self.text_edited_slot)
-            
-    def focusInEvent(self, ev):
-        self.setPalette(QApplication.palette(self))
-        if self.in_help_mode():
-            self.setText('')
-        return QLineEdit.focusInEvent(self, ev)
-    
-    def in_help_mode(self):
-        return unicode(self.text()) == self.HELP_TEXT
-    
-    def clear_to_help_mode(self):
-        self.setPalette(self.gray)
-        self.setText(self.HELP_TEXT)
-        
-    def text_edited_slot(self, text):
-        text = unicode(text)
-        self.timer = self.startTimer(self.INTERVAL)
-
-    def timerEvent(self, event):
-        self.killTimer(event.timerId())
-        if event.timerId() == self.timer:
-            text = unicode(self.text())
-            self.emit(SIGNAL('search(PyQt_PyObject)'), text)
 
 def encode_schedule(day, hour, minute):
     day = 1e7 * (day+1)
@@ -279,11 +251,12 @@ def decode_schedule(num):
     return day-1, hour-1, minute-1
 
 class SchedulerDialog(QDialog, Ui_Dialog):
-    
+
     def __init__(self, db, *args):
         QDialog.__init__(self, *args)
         self.setupUi(self)
-        self.search = Search(self)
+        self.search = SearchBox2(self)
+        self.search.initialize('scheduler_search_history')
         self.recipe_box.layout().insertWidget(0, self.search)
         self.detail_box.setVisible(False)
         self._model = RecipeModel(db)
@@ -300,31 +273,39 @@ class SchedulerDialog(QDialog, Ui_Dialog):
         self.connect(self.time, SIGNAL('timeChanged(QTime)'), self.do_schedule)
         for button in (self.daily_button, self.interval_button):
             self.connect(button, SIGNAL('toggled(bool)'), self.do_schedule)
-        self.connect(self.search, SIGNAL('search(PyQt_PyObject)'), self._model.search)
+        self.connect(self.search, SIGNAL('search(PyQt_PyObject,PyQt_PyObject)'), self._model.search)
+        self.connect(self._model,  SIGNAL('searched(PyQt_PyObject)'),
+                self.search.search_done)
         self.connect(self._model, SIGNAL('modelReset()'), lambda : self.detail_box.setVisible(False))
+        self.connect(self.download_all_button, SIGNAL('clicked()'),
+                self.download_all)
         self.connect(self.download, SIGNAL('clicked()'), self.download_now)
         self.search.setFocus(Qt.OtherFocusReason)
         self.old_news.setValue(gconf['oldest_news'])
         self.rnumber.setText(_('%d recipes')%self._model.num_of_recipes)
-        for day in (_('day'), _('Monday'), _('Tuesday'), _('Wednesday'), 
+        for day in (_('day'), _('Monday'), _('Tuesday'), _('Wednesday'),
                     _('Thursday'), _('Friday'), _('Saturday'), _('Sunday')):
             self.day.addItem(day)
-    
+
     def currentChanged(self, current, previous):
         if current.parent().isValid():
             self.show_recipe(current)
-        
+
+    def download_all(self, *args):
+        for recipe in self._model.scheduled_recipes():
+            self.emit(SIGNAL('download_now(PyQt_PyObject)'), recipe)
+
     def download_now(self):
         recipe = self._model.data(self.recipes.currentIndex(), Qt.UserRole)
         self.emit(SIGNAL('download_now(PyQt_PyObject)'), recipe)
-        
+
     def set_account_info(self, *args):
         username, password = map(unicode, (self.username.text(), self.password.text()))
         username, password = username.strip(), password.strip()
         recipe = self._model.data(self.recipes.currentIndex(), Qt.UserRole)
         key = 'recipe_account_info_%s'%recipe.id
-        config[key] = (username, password) if username and password else None
-        
+        config[key] = (username, password) if username else None
+
     def do_schedule(self, *args):
         if not getattr(self, 'allow_scheduling', False):
             return
@@ -340,7 +321,7 @@ class SchedulerDialog(QDialog, Ui_Dialog):
                 recipe.last_downloaded = datetime.fromordinal(1)
                 recipes.append(recipe)
             if recipe.needs_subscription and not config['recipe_account_info_%s'%recipe.id]:
-                error_dialog(self, _('Must set account information'), 
+                error_dialog(self, _('Must set account information'),
                              _('This recipe requires a username and password')).exec_()
                 self.schedule.setCheckState(Qt.Unchecked)
                 return
@@ -362,7 +343,7 @@ class SchedulerDialog(QDialog, Ui_Dialog):
         save_recipes(recipes)
         self._model.update_recipe_schedule(recipe)
         self.emit(SIGNAL('new_schedule(PyQt_PyObject)'), recipes)
-                
+
     def show_recipe(self, index):
         recipe = self._model.data(index, Qt.UserRole)
         self.current_recipe = recipe
@@ -393,7 +374,7 @@ class SchedulerDialog(QDialog, Ui_Dialog):
             self.interval_button.setChecked(False)
             self.interval.setEnabled(False)
         self.schedule.setChecked(recipe.schedule is not None)
-        self.allow_scheduling = True    
+        self.allow_scheduling = True
         self.detail_box.setVisible(True)
         self.account.setVisible(recipe.needs_subscription)
         self.interval.setEnabled(self.schedule.checkState() == Qt.Checked)
@@ -415,11 +396,11 @@ class SchedulerDialog(QDialog, Ui_Dialog):
             self.last_downloaded.setText(_('Last downloaded')+': '+tm)
         else:
             self.last_downloaded.setText(_('Last downloaded: never'))
-            
+
 class Scheduler(QObject):
-    
+
     INTERVAL = 1 # minutes
-    
+
     def __init__(self, main):
         self.main = main
         self.verbose = main.verbose
@@ -437,36 +418,36 @@ class Scheduler(QObject):
         self.oldest = gconf['oldest_news']
         self.oldest_timer.start(int(60 * 60000))
         self.oldest_check()
-        
+
         self.news_menu = QMenu()
-        self.news_icon = QIcon(':/images/news.svg')
-        self.scheduler_action = QAction(QIcon(':/images/scheduler.svg'), _('Schedule news download'), self)
+        self.news_icon = QIcon(I('news.svg'))
+        self.scheduler_action = QAction(QIcon(I('scheduler.svg')), _('Schedule news download'), self)
         self.news_menu.addAction(self.scheduler_action)
         self.connect(self.scheduler_action, SIGNAL('triggered(bool)'), self.show_dialog)
-        self.cac = QAction(QIcon(':/images/user_profile.svg'), _('Add a custom news source'), self)
+        self.cac = QAction(QIcon(I('user_profile.svg')), _('Add a custom news source'), self)
         self.connect(self.cac, SIGNAL('triggered(bool)'), self.customize_feeds)
         self.news_menu.addAction(self.cac)
-        
+
     def oldest_check(self):
         if self.oldest > 0:
             delta = timedelta(days=self.oldest)
             ids = self.main.library_view.model().db.tags_older_than(_('News'), delta)
             if ids:
                 self.main.library_view.model().delete_books_by_id(ids)
-    
+
     def customize_feeds(self, *args):
         main = self.main
         d = UserProfiles(main, main.library_view.model().db.get_feeds())
         d.exec_()
         feeds = tuple(d.profiles())
         main.library_view.model().db.set_feeds(feeds)
-            
-    
+
+
     def debug(self, *args):
         if self.verbose:
             sys.stdout.write(' '.join(map(unicode, args))+'\n')
             sys.stdout.flush()
-    
+
     def check(self):
         if not self.lock.tryLock():
             return
@@ -492,15 +473,15 @@ class Scheduler(QObject):
                     matches = day_matches and (hour*60+minute) < tnow
                     if matches and recipe.last_downloaded.toordinal() < date.today().toordinal():
                         needs_downloading.add(recipe)
-                
+
             self.debug('Needs downloading:', needs_downloading)
-        
+
             needs_downloading = [r for r in needs_downloading if r not in self.queue]
             for recipe in needs_downloading:
                 self.do_download(recipe)
         finally:
             self.lock.unlock()
-            
+
     def do_download(self, recipe):
         try:
             id = int(recipe.id)
@@ -509,8 +490,12 @@ class Scheduler(QObject):
                 self.recipes.remove(recipe)
                 save_recipes(self.recipes)
                 return
+            pt = PersistentTemporaryFile('_builtin.recipe')
+            pt.write(script)
+            pt.close()
+            script = pt.name
         except ValueError:
-            script = recipe.title
+            script = recipe.title + '.recipe'
         self.debug('\tQueueing:', recipe)
         self.main.download_scheduled_recipe(recipe, script, self.recipe_downloaded)
         self.queue.add(recipe)
@@ -536,7 +521,7 @@ class Scheduler(QObject):
         finally:
             self.lock.unlock()
         self.debug('Downloaded:', recipe)
-            
+
     def download(self, recipe):
         self.lock.lock()
         try:
@@ -546,10 +531,10 @@ class Scheduler(QObject):
                 self.do_download(recipe)
         finally:
             self.lock.unlock()
-    
+
     def refresh_schedule(self, recipes):
         self.recipes = recipes
-    
+
     def show_dialog(self, *args):
         self.lock.lock()
         try:
