@@ -8,19 +8,22 @@ from operator import attrgetter
 from math import cos, sin, pi
 from PyQt4.QtGui import QTableView, QAbstractItemView, QColor, \
                         QItemDelegate, QPainterPath, QLinearGradient, QBrush, \
-                        QPen, QStyle, QPainter, QLineEdit, \
-                        QPalette, QImage, QApplication, QMenu, QStyledItemDelegate
-from PyQt4.QtCore import QAbstractTableModel, QVariant, Qt, QString, \
+                        QPen, QStyle, QPainter, \
+                        QImage, QApplication, QMenu, \
+                        QStyledItemDelegate, QCompleter
+from PyQt4.QtCore import QAbstractTableModel, QVariant, Qt, \
                          SIGNAL, QObject, QSize, QModelIndex, QDate
 
 from calibre import strftime
 from calibre.ptempfile import PersistentTemporaryFile
+from calibre.utils.pyparsing import ParseException
 from calibre.library.database2 import FIELD_MAP
 from calibre.gui2 import NONE, TableView, qstring_to_unicode, config, \
                          error_dialog
+from calibre.gui2.widgets import EnLineEdit, TagsLineEdit
 from calibre.utils.search_query_parser import SearchQueryParser
 from calibre.ebooks.metadata.meta import set_metadata as _set_metadata
-from calibre.ebooks.metadata import string_to_authors
+from calibre.ebooks.metadata import string_to_authors, fmt_sidx
 
 class LibraryDelegate(QItemDelegate):
     COLOR    = QColor("blue")
@@ -86,51 +89,90 @@ class LibraryDelegate(QItemDelegate):
         return sb
 
 class DateDelegate(QStyledItemDelegate):
-    
+
     def displayText(self, val, locale):
         d = val.toDate()
         return d.toString('dd MMM yyyy')
-        
+
     def createEditor(self, parent, option, index):
         qde = QStyledItemDelegate.createEditor(self, parent, option, index)
-        qde.setDisplayFormat('MM/dd/yyyy')
+        qde.setDisplayFormat(unicode(qde.displayFormat()).replace('yy', 'yyyy'))
+        qde.setMinimumDate(QDate(101,1,1))
+        qde.setCalendarPopup(True)
         return qde
 
+class PubDateDelegate(QStyledItemDelegate):
+
+    def displayText(self, val, locale):
+        return val.toDate().toString('MMM yyyy')
+
+    def createEditor(self, parent, option, index):
+        qde = QStyledItemDelegate.createEditor(self, parent, option, index)
+        qde.setDisplayFormat('MM yyyy')
+        qde.setMinimumDate(QDate(101,1,1))
+        qde.setCalendarPopup(True)
+        return qde
+
+class TextDelegate(QStyledItemDelegate):
+
+    def __init__(self, parent):
+        '''
+        Delegate for text data. If auto_complete_function needs to return a list
+        of text items to auto-complete with. The funciton is None no
+        auto-complete will be used.
+        '''
+        QStyledItemDelegate.__init__(self, parent)
+        self.auto_complete_function = None
+
+    def set_auto_complete_function(self, f):
+        self.auto_complete_function = f
+
+    def createEditor(self, parent, option, index):
+        editor = EnLineEdit(parent)
+        if self.auto_complete_function:
+            complete_items = [i[1] for i in self.auto_complete_function()]
+            completer = QCompleter(complete_items, self)
+            completer.setCaseSensitivity(Qt.CaseInsensitive)
+            completer.setCompletionMode(QCompleter.InlineCompletion)
+            editor.setCompleter(completer)
+        return editor
+
+class TagsDelegate(QStyledItemDelegate):
+
+    def __init__(self, parent):
+        QStyledItemDelegate.__init__(self, parent)
+        self.db = None
+
+    def set_database(self, db):
+        self.db = db
+
+    def createEditor(self, parent, option, index):
+        if self.db:
+            editor = TagsLineEdit(parent, self.db.all_tags())
+        else:
+            editor = EnLineEdit(parent)
+        return editor
+
 class BooksModel(QAbstractTableModel):
-    coding = zip(
-    [1000,900,500,400,100,90,50,40,10,9,5,4,1],
-    ["M","CM","D","CD","C","XC","L","XL","X","IX","V","IV","I"]
-    )
-    
     headers = {
                         'title'     : _("Title"),
                         'authors'   : _("Author(s)"),
                         'size'      : _("Size (MB)"),
                         'timestamp' : _("Date"),
+                        'pubdate'   : _('Published'),
                         'rating'    : _('Rating'),
                         'publisher' : _("Publisher"),
                         'tags'      : _("Tags"),
                         'series'    : _("Series"),
                         }
-            
-    @classmethod
-    def roman(cls, num):
-        if num <= 0 or num >= 4000 or int(num) != num:
-            return str(num)
-        result = []
-        for d, r in cls.coding:
-            while num >= d:
-                result.append(r)
-                num -= d
-        return ''.join(result)
 
     def __init__(self, parent=None, buffer=40):
         QAbstractTableModel.__init__(self, parent)
         self.db = None
         self.column_map = config['column_map']
-        self.editable_cols = ['title', 'authors', 'rating', 'publisher', 
-                              'tags', 'series', 'timestamp']
-        self.default_image = QImage(':/images/book.svg')
+        self.editable_cols = ['title', 'authors', 'rating', 'publisher',
+                              'tags', 'series', 'timestamp', 'pubdate']
+        self.default_image = QImage(I('book.svg'))
         self.sorted_on = ('timestamp', Qt.AscendingOrder)
         self.last_search = '' # The last search performed on this model
         self.read_config()
@@ -147,18 +189,8 @@ class BooksModel(QAbstractTableModel):
         if cols != self.column_map:
             self.column_map = cols
             self.reset()
-            try:
-                idx = self.column_map.index('rating')
-            except ValueError:
-                idx = -1
-            try:
-                tidx = self.column_map.index('timestamp')
-            except ValueError:
-                tidx = -1
-            
-            self.emit(SIGNAL('columns_sorted(int,int)'), idx, tidx)
-        
-    
+            self.emit(SIGNAL('columns_sorted()'))
+
     def set_database(self, db):
         self.db = db
         self.build_data_convertors()
@@ -167,7 +199,7 @@ class BooksModel(QAbstractTableModel):
         rows = self.db.refresh_ids(ids)
         if rows:
             self.refresh_rows(rows, current_row=current_row)
-            
+
     def refresh_rows(self, rows, current_row=-1):
         for row in rows:
             if self.cover_cache:
@@ -184,17 +216,17 @@ class BooksModel(QAbstractTableModel):
         self.db = None
         self.reset()
 
-    def add_books(self, paths, formats, metadata, uris=[], add_duplicates=False):
-        ret = self.db.add_books(paths, formats, metadata, uris,
+    def add_books(self, paths, formats, metadata, add_duplicates=False):
+        ret = self.db.add_books(paths, formats, metadata,
                                  add_duplicates=add_duplicates)
         self.count_changed()
         return ret
-        
+
     def add_news(self, path, recipe):
         ret = self.db.add_news(path, recipe)
         self.count_changed()
         return ret
-    
+
     def count_changed(self, *args):
         self.emit(SIGNAL('count_changed(int)'), self.db.count())
 
@@ -202,19 +234,9 @@ class BooksModel(QAbstractTableModel):
         ''' Return list indices of all cells in index.row()'''
         return [ self.index(index.row(), c) for c in range(self.columnCount(None))]
 
-    def save_to_disk(self, rows, path, single_dir=False, single_format=None,
-                     callback=None):
-        rows = [row.row() for row in rows]
-        if single_format is None:
-            return self.db.export_to_dir(path, rows, 
-                                         self.sorted_on[0] == 'authors', 
-                                         single_dir=single_dir, 
-                                         callback=callback)
-        else:
-            return self.db.export_single_format_to_dir(path, rows, 
-                                                       single_format,
-                                                       callback=callback)
-
+    @property
+    def by_author(self):
+        return self.sorted_on[0] == 'authors'
 
     def delete_books(self, indices):
         ids = map(self.id, indices)
@@ -223,8 +245,8 @@ class BooksModel(QAbstractTableModel):
         self.count_changed()
         self.clear_caches()
         self.reset()
-        
-        
+
+
     def delete_books_by_id(self, ids):
         for id in ids:
             try:
@@ -246,11 +268,18 @@ class BooksModel(QAbstractTableModel):
             self.count_changed()
 
     def search(self, text, refinement, reset=True):
-        self.db.search(text)
+        try:
+            self.db.search(text)
+        except ParseException:
+            self.emit(SIGNAL('searched(PyQt_PyObject)'), False)
+            return
         self.last_search = text
         if reset:
             self.clear_caches()
             self.reset()
+        if self.last_search:
+            self.emit(SIGNAL('searched(PyQt_PyObject)'), True)
+
 
     def sort(self, col, order, reset=True):
         if not self.db:
@@ -261,18 +290,18 @@ class BooksModel(QAbstractTableModel):
             self.clear_caches()
             self.reset()
         self.sorted_on = (self.column_map[col], order)
-        
-    
+
+
     def refresh(self, reset=True):
         try:
             col = self.column_map.index(self.sorted_on[0])
         except:
             col = 0
-        self.db.refresh(field=self.column_map[col], 
+        self.db.refresh(field=self.column_map[col],
                         ascending=self.sorted_on[1]==Qt.AscendingOrder)
         if reset:
             self.reset()
-    
+
     def resort(self, reset=True):
         try:
             col = self.column_map.index(self.sorted_on[0])
@@ -321,7 +350,7 @@ class BooksModel(QAbstractTableModel):
         series = self.db.series(idx)
         if series:
             sidx = self.db.series_index(idx)
-            sidx = self.__class__.roman(sidx) if self.use_roman_numbers else str(sidx)
+            sidx = fmt_sidx(sidx, use_roman = self.use_roman_numbers)
             data[_('Series')] = _('Book <font face="serif">%s</font> of %s.')%(sidx, series)
 
         return data
@@ -395,31 +424,45 @@ class BooksModel(QAbstractTableModel):
         else:
             return metadata
 
-    def get_preferred_formats_from_ids(self, ids, all_formats, mode='r+b'):
+    def get_preferred_formats_from_ids(self, ids, formats, paths=False,
+                              set_metadata=False, specific_format=None,
+                              exclude_auto=False, mode='r+b'):
         ans = []
+        need_auto = []
+        if specific_format is not None:
+            formats = [specific_format.lower()]
         for id in ids:
             format = None
             fmts = self.db.formats(id, index_is_id=True)
             if not fmts:
                 fmts = ''
-            available_formats = set(fmts.lower().split(','))
-            for f in all_formats:
-                if f.lower() in available_formats:
-                    format = f.lower()
+            db_formats = set(fmts.lower().split(','))
+            available_formats = set([f.lower() for f in formats])
+            u = available_formats.intersection(db_formats)
+            for f in formats:
+                if f.lower() in u:
+                    format = f
                     break
-            if format is None:
-                ans.append(format)
+            if format is not None:
+                pt = PersistentTemporaryFile(suffix='.'+format)
+                pt.write(self.db.format(id, format, index_is_id=True))
+                pt.flush()
+                if set_metadata:
+                    _set_metadata(pt, self.db.get_metadata(id, get_cover=True, index_is_id=True),
+                                  format)
+                pt.close() if paths else pt.seek(0)
+                ans.append(pt)
             else:
-                f = self.db.format(id, format, index_is_id=True, as_file=True, 
-                                   mode=mode)
-                ans.append(f)
-        return ans
-                     
-            
-    
-    def get_preferred_formats(self, rows, formats, paths=False, 
-                              set_metadata=False, specific_format=None):
+                need_auto.append(id)
+                if not exclude_auto:
+                    ans.append(None)
+        return ans, need_auto
+
+    def get_preferred_formats(self, rows, formats, paths=False,
+                              set_metadata=False, specific_format=None,
+                              exclude_auto=False):
         ans = []
+        need_auto = []
         if specific_format is not None:
             formats = [specific_format.lower()]
         for row in (row.row() for row in rows):
@@ -428,7 +471,7 @@ class BooksModel(QAbstractTableModel):
             if not fmts:
                 fmts = ''
             db_formats = set(fmts.lower().split(','))
-            available_formats = set([f.lower() for f in formats]) 
+            available_formats = set([f.lower() for f in formats])
             u = available_formats.intersection(db_formats)
             for f in formats:
                 if f.lower() in u:
@@ -444,8 +487,10 @@ class BooksModel(QAbstractTableModel):
                 pt.close() if paths else pt.seek(0)
                 ans.append(pt)
             else:
-                ans.append(None)
-        return ans
+                need_auto.append(row)
+                if not exclude_auto:
+                    ans.append(None)
+        return ans, need_auto
 
     def id(self, row):
         return self.db.id(getattr(row, 'row', lambda:row)())
@@ -467,7 +512,7 @@ class BooksModel(QAbstractTableModel):
                 data = self.db.cover(row_number)
         except IndexError: # Happens if database has not yet been refreshed
             pass
-        
+
         if not data:
             return self.default_image
         img = QImage()
@@ -477,63 +522,71 @@ class BooksModel(QAbstractTableModel):
         return img
 
     def build_data_convertors(self):
-        
+
         tidx = FIELD_MAP['title']
         aidx = FIELD_MAP['authors']
         sidx = FIELD_MAP['size']
         ridx = FIELD_MAP['rating']
         pidx = FIELD_MAP['publisher']
         tmdx = FIELD_MAP['timestamp']
+        pddx = FIELD_MAP['pubdate']
         srdx = FIELD_MAP['series']
         tgdx = FIELD_MAP['tags']
         siix = FIELD_MAP['series_index']
-        
+
         def authors(r):
             au = self.db.data[r][aidx]
             if au:
                 au = [a.strip().replace('|', ',') for a in au.split(',')]
                 return ' & '.join(au)
-            
+
         def timestamp(r):
             dt = self.db.data[r][tmdx]
             if dt:
                 dt = dt - timedelta(seconds=time.timezone) + timedelta(hours=time.daylight)
                 return QDate(dt.year, dt.month, dt.day)
-        
+
+        def pubdate(r):
+            dt = self.db.data[r][pddx]
+            if dt:
+                dt = dt - timedelta(seconds=time.timezone) + timedelta(hours=time.daylight)
+                return QDate(dt.year, dt.month, dt.day)
+
         def rating(r):
             r = self.db.data[r][ridx]
             r = r/2 if r else 0
             return r
-            
+
         def publisher(r):
             pub = self.db.data[r][pidx]
             if pub:
                 return pub
-        
+
         def tags(r):
             tags = self.db.data[r][tgdx]
             if tags:
                 return ', '.join(tags.split(','))
-        
+
         def series(r):
             series = self.db.data[r][srdx]
             if series:
-                return series  + ' [%d]'%self.db.data[r][siix]
-            
+                idx = fmt_sidx(self.db.data[r][siix])
+                return series + ' [%s]'%idx
         def size(r):
             size = self.db.data[r][sidx]
             if size:
                 return '%.1f'%(float(size)/(1024*1024))
-        
+
         self.dc = {
                    'title'    : lambda r : self.db.data[r][tidx],
                    'authors'  : authors,
                    'size'     : size,
                    'timestamp': timestamp,
+                   'pubdate' : pubdate,
                    'rating'   : rating,
                    'publisher': publisher,
                    'tags'     : tags,
-                   'series'   : series,                                                                       
+                   'series'   : series,
                    }
 
     def data(self, index, role):
@@ -569,18 +622,18 @@ class BooksModel(QAbstractTableModel):
             if column not in self.editable_cols:
                 return False
             val = int(value.toInt()[0]) if column == 'rating' else \
-                  value.toDate() if column == 'timestamp' else \
+                  value.toDate() if column in ('timestamp', 'pubdate') else \
                   unicode(value.toString())
-            id = self.db.id(row)  
+            id = self.db.id(row)
             if column == 'rating':
                 val = 0 if val < 0 else 5 if val > 5 else val
                 val *= 2
                 self.db.set_rating(id, val)
             elif column == 'series':
-                pat = re.compile(r'\[(\d+)\]')
+                pat = re.compile(r'\[([.0-9]+)\]')
                 match = pat.search(val)
                 if match is not None:
-                    self.db.set_series_index(id, int(match.group(1)))
+                    self.db.set_series_index(id, float(match.group(1)))
                     val = pat.sub('', val)
                 val = val.strip()
                 if val:
@@ -590,13 +643,18 @@ class BooksModel(QAbstractTableModel):
                     return False
                 dt = datetime(val.year(), val.month(), val.day()) + timedelta(seconds=time.timezone) - timedelta(hours=time.daylight)
                 self.db.set_timestamp(id, dt)
+            elif column == 'pubdate':
+                if val.isNull() or not val.isValid():
+                    return False
+                dt = datetime(val.year(), val.month(), val.day()) + timedelta(seconds=time.timezone) - timedelta(hours=time.daylight)
+                self.db.set_pubdate(id, dt)
             else:
                 self.db.set(row, column, val)
             self.emit(SIGNAL("dataChanged(QModelIndex, QModelIndex)"), \
                                 index, index)
             if column == self.sorted_on[0]:
                 self.resort()
-            
+
         return True
 
 class BooksView(TableView):
@@ -617,30 +675,48 @@ class BooksView(TableView):
         TableView.__init__(self, parent)
         self.rating_delegate = LibraryDelegate(self)
         self.timestamp_delegate = DateDelegate(self)
+        self.pubdate_delegate = PubDateDelegate(self)
+        self.tags_delegate = TagsDelegate(self)
+        self.authors_delegate = TextDelegate(self)
+        self.series_delegate = TextDelegate(self)
+        self.publisher_delegate = TextDelegate(self)
         self.display_parent = parent
         self._model = modelcls(self)
         self.setModel(self._model)
         self.setSelectionBehavior(QAbstractItemView.SelectRows)
         self.setSortingEnabled(True)
-        try:
-            self.columns_sorted(self._model.column_map.index('rating'),
-                                self._model.column_map.index('timestamp'))
-        except ValueError:
-            pass
+        for i in range(10):
+            self.setItemDelegateForColumn(i, TextDelegate(self))
+        self.columns_sorted()
         QObject.connect(self.selectionModel(), SIGNAL('currentRowChanged(QModelIndex, QModelIndex)'),
                         self._model.current_changed)
-        self.connect(self._model, SIGNAL('columns_sorted(int, int)'), self.columns_sorted, Qt.QueuedConnection)
-        
-    def columns_sorted(self, rating_col, timestamp_col):
+        self.connect(self._model, SIGNAL('columns_sorted()'),
+                self.columns_sorted, Qt.QueuedConnection)
+
+    def columns_sorted(self):
         for i in range(self.model().columnCount(None)):
-            if self.itemDelegateForColumn(i) == self.rating_delegate:
+            if self.itemDelegateForColumn(i) in (self.rating_delegate,
+                    self.timestamp_delegate, self.pubdate_delegate):
                 self.setItemDelegateForColumn(i, self.itemDelegate())
-        if rating_col > -1:
-            self.setItemDelegateForColumn(rating_col, self.rating_delegate)
-        if timestamp_col > -1:
-            self.setItemDelegateForColumn(timestamp_col, self.timestamp_delegate)
-            
-    def set_context_menu(self, edit_metadata, send_to_device, convert, view, 
+
+        cm = self._model.column_map
+
+        if 'rating' in cm:
+            self.setItemDelegateForColumn(cm.index('rating'), self.rating_delegate)
+        if 'timestamp' in cm:
+            self.setItemDelegateForColumn(cm.index('timestamp'), self.timestamp_delegate)
+        if 'pubdate' in cm:
+            self.setItemDelegateForColumn(cm.index('pubdate'), self.pubdate_delegate)
+        if 'tags' in cm:
+            self.setItemDelegateForColumn(cm.index('tags'), self.tags_delegate)
+        if 'authors' in cm:
+            self.setItemDelegateForColumn(cm.index('authors'), self.authors_delegate)
+        if 'publisher' in cm:
+            self.setItemDelegateForColumn(cm.index('publisher'), self.publisher_delegate)
+        if 'series' in cm:
+            self.setItemDelegateForColumn(cm.index('series'), self.series_delegate)
+
+    def set_context_menu(self, edit_metadata, send_to_device, convert, view,
                          save, open_folder, book_details, similar_menu=None):
         self.setContextMenuPolicy(Qt.DefaultContextMenu)
         self.context_menu = QMenu(self)
@@ -658,18 +734,18 @@ class BooksView(TableView):
             self.context_menu.addAction(book_details)
         if similar_menu is not None:
             self.context_menu.addMenu(similar_menu)
-        
+
     def contextMenuEvent(self, event):
         self.context_menu.popup(event.globalPos())
         event.accept()
-    
+
     def sortByColumn(self, colname, order):
         try:
             idx = self._model.column_map.index(colname)
         except ValueError:
             idx = 0
         TableView.sortByColumn(self, idx, order)
-    
+
     @classmethod
     def paths_from_event(cls, event):
         '''
@@ -696,23 +772,36 @@ class BooksView(TableView):
         paths = self.paths_from_event(event)
         event.setDropAction(Qt.CopyAction)
         event.accept()
-        self.emit(SIGNAL('files_dropped(PyQt_PyObject)'), paths, Qt.QueuedConnection)
+        self.emit(SIGNAL('files_dropped(PyQt_PyObject)'), paths)
 
 
     def set_database(self, db):
         self._model.set_database(db)
+        self.tags_delegate.set_database(db)
+        self.authors_delegate.set_auto_complete_function(db.all_authors)
+        self.series_delegate.set_auto_complete_function(db.all_series)
+        self.publisher_delegate.set_auto_complete_function(db.all_publishers)
 
     def close(self):
         self._model.close()
 
-    def connect_to_search_box(self, sb):
+    def set_editable(self, editable):
+        self._model.set_editable(editable)
+
+    def connect_to_search_box(self, sb, search_done):
         QObject.connect(sb, SIGNAL('search(PyQt_PyObject, PyQt_PyObject)'),
                         self._model.search)
+        self._search_done = search_done
+        self.connect(self._model, SIGNAL('searched(PyQt_PyObject)'),
+                self.search_done)
 
     def connect_to_book_display(self, bd):
         QObject.connect(self._model, SIGNAL('new_bookdisplay_data(PyQt_PyObject)'),
                         bd)
 
+    def search_done(self, ok):
+        self._search_done(self, ok)
+
 
 class DeviceBooksView(BooksView):
 
@@ -722,44 +811,48 @@ class DeviceBooksView(BooksView):
         self.resize_on_select = False
         self.rating_delegate = None
         for i in range(10):
-            self.setItemDelegateForColumn(i, self.itemDelegate())
+            self.setItemDelegateForColumn(i, TextDelegate(self))
         self.setDragDropMode(self.NoDragDrop)
         self.setAcceptDrops(False)
 
+    def set_database(self, db):
+        self._model.set_database(db)
+
     def resizeColumnsToContents(self):
         QTableView.resizeColumnsToContents(self)
         self.columns_resized = True
 
     def connect_dirtied_signal(self, slot):
         QObject.connect(self._model, SIGNAL('booklist_dirtied()'), slot)
-        
+
     def sortByColumn(self, col, order):
         TableView.sortByColumn(self, col, order)
-        
+
     def dropEvent(self, *args):
-        error_dialog(self, _('Not allowed'), 
+        error_dialog(self, _('Not allowed'),
         _('Dropping onto a device is not supported. First add the book to the calibre library.')).exec_()
 
 class OnDeviceSearch(SearchQueryParser):
-    
+
     def __init__(self, model):
         SearchQueryParser.__init__(self)
         self.model = model
-        
+
     def universal_set(self):
         return set(range(0, len(self.model.db)))
-    
+
     def get_matches(self, location, query):
         location = location.lower().strip()
         query = query.lower().strip()
-        if location not in ('title', 'authors', 'tags', 'all'):
+        if location not in ('title', 'author', 'tag', 'all', 'format'):
             return set([])
         matches = set([])
-        locations = ['title', 'authors', 'tags'] if location == 'all' else [location]
+        locations = ['title', 'author', 'tag', 'format'] if location == 'all' else [location]
         q = {
              'title' : lambda x : getattr(x, 'title').lower(),
-             'authors': lambda x: getattr(x, 'authors').lower(),
-             'tags':lambda x: ','.join(getattr(x, 'tags')).lower()
+             'author': lambda x: getattr(x, 'authors').lower(),
+             'tag':lambda x: ','.join(getattr(x, 'tags')).lower(),
+             'format':lambda x: os.path.splitext(x.path)[1].lower()
              }
         for i, v in enumerate(locations):
             locations[i] = q[v]
@@ -769,7 +862,7 @@ class OnDeviceSearch(SearchQueryParser):
                     matches.add(i)
                     break
         return matches
-        
+
 
 class DeviceBooksModel(BooksModel):
 
@@ -778,10 +871,10 @@ class DeviceBooksModel(BooksModel):
         self.db  = []
         self.map = []
         self.sorted_map = []
-        self.unknown = str(self.trUtf8('Unknown'))
+        self.unknown = _('Unknown')
         self.marked_for_deletion = {}
         self.search_engine = OnDeviceSearch(self)
-
+        self.editable = True
 
     def mark_for_deletion(self, job, rows):
         self.marked_for_deletion[job] = self.indices(rows)
@@ -789,7 +882,6 @@ class DeviceBooksModel(BooksModel):
             indices = self.row_indices(row)
             self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), indices[0], indices[-1])
 
-
     def deletion_done(self, job, succeeded=True):
         if not self.marked_for_deletion.has_key(job):
             return
@@ -814,17 +906,22 @@ class DeviceBooksModel(BooksModel):
         if self.map[index.row()] in self.indices_to_be_deleted():
             return Qt.ItemIsUserCheckable  # Can't figure out how to get the disabled flag in python
         flags = QAbstractTableModel.flags(self, index)
-        if index.isValid():
+        if index.isValid() and self.editable:
             if index.column() in [0, 1] or (index.column() == 4 and self.db.supports_tags()):
                 flags |= Qt.ItemIsEditable
         return flags
 
 
     def search(self, text, refinement, reset=True):
-        if not text:
+        if not text or not text.strip():
             self.map = list(range(len(self.db)))
         else:
-            matches = self.search_engine.parse(text)
+            try:
+                matches = self.search_engine.parse(text)
+            except ParseException:
+                self.emit(SIGNAL('searched(PyQt_PyObject)'), False)
+                return
+
             self.map = []
             for i in range(len(self.db)):
                 if i in matches:
@@ -833,10 +930,13 @@ class DeviceBooksModel(BooksModel):
         if reset:
             self.reset()
         self.last_search = text
-    
+        if self.last_search:
+            self.emit(SIGNAL('searched(PyQt_PyObject)'), True)
+
+
     def resort(self, reset):
         self.sort(self.sorted_on[0], self.sorted_on[1], reset=reset)
-    
+
     def sort(self, col, order, reset=True):
         descending = order != Qt.AscendingOrder
         def strcmp(attr):
@@ -955,7 +1055,7 @@ class DeviceBooksModel(BooksModel):
                 return QVariant('Marked for deletion')
             col = index.column()
             if col in [0, 1] or (col == 4 and self.db.supports_tags()):
-                return QVariant("Double click to <b>edit</b> me<br><br>")
+                return QVariant(_("Double click to <b>edit</b> me<br><br>"))
         return NONE
 
     def headerData(self, section, orientation, role):
@@ -996,76 +1096,8 @@ class DeviceBooksModel(BooksModel):
             done = True
         return done
 
-class SearchBox(QLineEdit):
-
-    INTERVAL = 1000 #: Time to wait before emitting search signal
-
-    def __init__(self, parent, help_text=_('Search (For Advanced Search click the button to the left)')):
-        QLineEdit.__init__(self, parent)
-        self.help_text = help_text
-        self.initial_state = True
-        self.default_palette = QApplication.palette(self)
-        self.gray = QPalette(self.default_palette)
-        self.gray.setBrush(QPalette.Text, QBrush(QColor('gray')))
-        self.prev_search = ''
-        self.timer = None
-        self.clear_to_help()
-        QObject.connect(self, SIGNAL('textEdited(QString)'), self.text_edited_slot)
+    def set_editable(self, editable):
+        self.editable = editable
 
 
-    def normalize_state(self):
-        self.setText('')
-        self.setPalette(self.default_palette)
 
-    def clear_to_help(self):
-        self.setPalette(self.gray)
-        self.setText(self.help_text)
-        self.home(False)
-        self.initial_state = True
-
-    def clear(self):
-        self.clear_to_help()
-        self.emit(SIGNAL('search(PyQt_PyObject, PyQt_PyObject)'), '', False)
-
-
-    def keyPressEvent(self, event):
-        if self.initial_state:
-            self.normalize_state()
-            self.initial_state = False
-        QLineEdit.keyPressEvent(self, event)
-
-    def mouseReleaseEvent(self, event):
-        if self.initial_state:
-            self.normalize_state()
-            self.initial_state = False
-        QLineEdit.mouseReleaseEvent(self, event)
-
-    def text_edited_slot(self, text):
-        text = qstring_to_unicode(text) if isinstance(text, QString) else unicode(text)
-        self.prev_text = text
-        self.timer = self.startTimer(self.__class__.INTERVAL)
-
-    def timerEvent(self, event):
-        self.killTimer(event.timerId())
-        if event.timerId() == self.timer:
-            text = qstring_to_unicode(self.text())
-            refinement = text.startswith(self.prev_search) and ':' not in text
-            self.prev_search = text
-            self.emit(SIGNAL('search(PyQt_PyObject, PyQt_PyObject)'), text, refinement)
-
-    def search_from_tokens(self, tokens, all):
-        ans = u' '.join([u'%s:%s'%x for x in tokens])
-        if not all:
-            ans = '[' + ans + ']'
-        self.set_search_string(ans)
-        
-    def search_from_tags(self, tags, all):
-        joiner = ' and ' if all else ' or '
-        self.set_search_string(joiner.join(tags))
-    
-    def set_search_string(self, txt):
-        self.normalize_state()
-        self.setText(txt)
-        self.emit(SIGNAL('search(PyQt_PyObject, PyQt_PyObject)'), txt, False)
-        self.end(False)
-        self.initial_state = False
