@@ -11,12 +11,8 @@ import sys, os, cStringIO
 from textwrap import TextWrapper
 from urllib import quote
 
-from calibre import terminal_controller, preferred_encoding
+from calibre import terminal_controller, preferred_encoding, prints
 from calibre.utils.config import OptionParser, prefs
-try:
-    from calibre.utils.single_qt_application import send_message
-except:
-    send_message = None
 from calibre.ebooks.metadata.meta import get_metadata
 from calibre.library.database2 import LibraryDatabase2
 from calibre.ebooks.metadata.opf2 import OPFCreator, OPF
@@ -55,7 +51,7 @@ XML_TEMPLATE = '''\
         </py:for>
         </formats>
     </record>
-</py:for> 
+</py:for>
 </calibredb>
 '''
 
@@ -101,6 +97,20 @@ STANZA_TEMPLATE='''\
 </feed>
 '''
 
+def send_message(msg=''):
+    prints('Notifying calibre of the change')
+    from calibre.utils.ipc import RC
+    import time
+    t = RC(print_error=False)
+    t.start()
+    time.sleep(3)
+    if t.done:
+        t.conn.send('refreshdb:'+msg)
+        t.conn.close()
+
+
+
+
 def get_parser(usage):
     parser = OptionParser(usage)
     go = parser.add_option_group('GLOBAL OPTIONS')
@@ -114,7 +124,7 @@ def get_db(dbpath, options):
     dbpath = os.path.abspath(dbpath)
     return LibraryDatabase2(dbpath)
 
-def do_list(db, fields, sort_by, ascending, search_text, line_width, separator, 
+def do_list(db, fields, sort_by, ascending, search_text, line_width, separator,
             prefix, output_format, subtitle='Books in the calibre database'):
     if sort_by:
         db.sort(sort_by, ascending)
@@ -125,7 +135,8 @@ def do_list(db, fields, sort_by, ascending, search_text, line_width, separator,
     fields = ['id'] + fields
     if output_format == 'text':
         for f in data:
-            f['formats'] = u'[%s]'%u','.join(f['formats'])
+            fmts = [x for x in f['formats'] if x is not None]
+            f['formats'] = u'[%s]'%u','.join(fmts)
         widths = list(map(lambda x : 0, fields))
         for record in data:
             for f in record.keys():
@@ -134,13 +145,13 @@ def do_list(db, fields, sort_by, ascending, search_text, line_width, separator,
         for i in data:
             for j, field in enumerate(fields):
                 widths[j] = max(widths[j], len(unicode(i[str(field)])))
-        
+
         screen_width = terminal_controller.COLS if line_width < 0 else line_width
         if not screen_width:
             screen_width = 80
         field_width = screen_width//len(fields)
         base_widths = map(lambda x: min(x+1, field_width), widths)
-    
+
         while sum(base_widths) < screen_width:
             adjusted = False
             for i in range(len(widths)):
@@ -150,14 +161,14 @@ def do_list(db, fields, sort_by, ascending, search_text, line_width, separator,
                     break
             if not adjusted:
                 break
-    
+
         widths = list(base_widths)
-        titles = map(lambda x, y: '%-*s'%(x, y), widths, fields)
+        titles = map(lambda x, y: '%-*s%s'%(x-len(separator), y, separator), widths, fields)
         print terminal_controller.GREEN + ''.join(titles)+terminal_controller.NORMAL
-    
+
         wrappers = map(lambda x: TextWrapper(x-1), widths)
         o = cStringIO.StringIO()
-    
+
         for record in data:
             text = [wrappers[i].wrap(unicode(record[field]).encode('utf-8')) for i, field in enumerate(fields)]
             lines = max(map(len, text))
@@ -178,12 +189,10 @@ def do_list(db, fields, sort_by, ascending, search_text, line_width, separator,
             if isinstance(x['fmt_epub'], unicode):
                 x['fmt_epub'] = x['fmt_epub'].encode('utf-8')
         template = MarkupTemplate(STANZA_TEMPLATE)
-        return template.generate(id="urn:calibre:main", data=data, subtitle=subtitle, 
+        return template.generate(id="urn:calibre:main", data=data, subtitle=subtitle,
                 sep=os.sep, quote=quote, updated=db.last_modified()).render('xml')
-            
 
-
-def command_list(args, dbpath):
+def list_option_parser():
     parser = get_parser(_(
 '''\
 %prog list [options]
@@ -199,13 +208,18 @@ List the books available in the calibre database.
                       help=_('Sort results in ascending order'))
     parser.add_option('-s', '--search', default=None,
                       help=_('Filter the results by the search query. For the format of the search query, please see the search related documentation in the User Manual. Default is to do no filtering.'))
-    parser.add_option('-w', '--line-width', default=-1, type=int, 
+    parser.add_option('-w', '--line-width', default=-1, type=int,
                       help=_('The maximum width of a single line in the output. Defaults to detecting screen size.'))
     parser.add_option('--separator', default=' ', help=_('The string used to separate fields. Default is a space.'))
     parser.add_option('--prefix', default=None, help=_('The prefix for all file paths. Default is the absolute path to the library folder.'))
     of = ['text', 'xml', 'stanza']
     parser.add_option('--output-format', choices=of, default='text',
                       help=_('The format in which to output the data. Available choices: %s. Defaults is text.')%of)
+    return parser
+
+
+def command_list(args, dbpath):
+    parser = list_option_parser()
     opts, args = parser.parse_args(sys.argv[:1] + args)
     fields = [str(f.strip().lower()) for f in opts.fields.split(',')]
     if 'all' in fields:
@@ -244,7 +258,10 @@ def do_add(db, paths, one_book_per_directory, recurse, add_duplicates):
             if os.path.isdir(path):
                 dirs.append(path)
             else:
-                files.append(path)
+                if os.path.exists(path):
+                    files.append(path)
+                else:
+                    print path, 'not found'
 
         formats, metadata = [], []
         for book in files:
@@ -262,11 +279,13 @@ def do_add(db, paths, one_book_per_directory, recurse, add_duplicates):
             formats.append(format)
             metadata.append(mi)
 
-        file_duplicates = db.add_books(files, formats, metadata, add_duplicates=add_duplicates)
-        if not file_duplicates[0]:
-            file_duplicates = []
-        else:
-            file_duplicates = file_duplicates[0]
+        file_duplicates = []
+        if files:
+            file_duplicates = db.add_books(files, formats, metadata,
+                                           add_duplicates=add_duplicates)
+            if file_duplicates:
+                file_duplicates = file_duplicates[0]
+
 
         dir_dups = []
         for dir in dirs:
@@ -286,7 +305,9 @@ def do_add(db, paths, one_book_per_directory, recurse, add_duplicates):
                 db.import_book(mi, formats)
         else:
             if dir_dups or file_duplicates:
-                print >>sys.stderr, _('The following books were not added as they already exist in the database (see --duplicates option):')
+                print >>sys.stderr, _('The following books were not added as '
+                                      'they already exist in the database '
+                                      '(see --duplicates option):')
             for mi, formats in dir_dups:
                 title = mi.title
                 if isinstance(title, unicode):
@@ -302,14 +323,11 @@ def do_add(db, paths, one_book_per_directory, recurse, add_duplicates):
                     print >>sys.stderr, '\t', title+':'
                     print >>sys.stderr, '\t\t ', path
 
-        if send_message is not None:
-            send_message('refreshdb:', 'calibre GUI')
+        send_message()
     finally:
         sys.stdout = orig
 
-
-
-def command_add(args, dbpath):
+def add_option_parser():
     parser = get_parser(_(
 '''\
 %prog add [options] file1 file2 file3 ...
@@ -324,6 +342,11 @@ the directory related options below.
                       help=_('Process directories recursively'))
     parser.add_option('-d', '--duplicates', action='store_true', default=False,
                       help=_('Add books to database even if they already exist. Comparison is done based on book titles.'))
+    return parser
+
+
+def command_add(args, dbpath):
+    parser = add_option_parser()
     opts, args = parser.parse_args(sys.argv[:1] + args)
     if len(args) < 2:
         parser.print_help()
@@ -341,12 +364,10 @@ def do_remove(db, ids):
             for y in x:
                 db.delete_book(y)
 
-    if send_message is not None:
-        send_message('refreshdb:', 'calibre GUI')
+        send_message()
 
-
-def command_remove(args, dbpath):
-    parser = get_parser(_(
+def remove_option_parser():
+    return get_parser(_(
 '''\
 %prog remove ids
 
@@ -354,6 +375,9 @@ Remove the books identified by ids from the database. ids should be a comma sepa
 list of id numbers (you can get id numbers by using the list command). For example, \
 23,34,57-85
 '''))
+
+def command_remove(args, dbpath):
+    parser = remove_option_parser()
     opts, args = parser.parse_args(sys.argv[:1] + args)
     if len(args) < 2:
         parser.print_help()
@@ -369,22 +393,25 @@ list of id numbers (you can get id numbers by using the list command). For examp
         else:
             ids.append(int(y[0]))
 
-    do_remove(get_db(dbpath, opts), ids)
+    do_remove(get_db(dbpath, opts), set(ids))
 
     return 0
 
 def do_add_format(db, id, fmt, path):
     db.add_format_with_hooks(id, fmt.upper(), path, index_is_id=True)
 
-
-def command_add_format(args, dbpath):
-    parser = get_parser(_(
+def add_format_option_parser():
+    return get_parser(_(
 '''\
 %prog add_format [options] id ebook_file
 
 Add the ebook in ebook_file to the available formats for the logical book identified \
 by id. You can get id by using the list command. If the format already exists, it is replaced.
 '''))
+
+
+def command_add_format(args, dbpath):
+    parser = add_format_option_parser()
     opts, args = parser.parse_args(sys.argv[:1] + args)
     if len(args) < 3:
         parser.print_help()
@@ -401,8 +428,8 @@ by id. You can get id by using the list command. If the format already exists, i
 def do_remove_format(db, id, fmt):
     db.remove_format(id, fmt, index_is_id=True)
 
-def command_remove_format(args, dbpath):
-    parser = get_parser(_(
+def remove_format_option_parser():
+    return get_parser(_(
 '''
 %prog remove_format [options] id fmt
 
@@ -411,6 +438,10 @@ You can get id by using the list command. fmt should be a file extension \
 like LRF or TXT or EPUB. If the logical book does not have fmt available, \
 do nothing.
 '''))
+
+
+def command_remove_format(args, dbpath):
+    parser = remove_format_option_parser()
     opts, args = parser.parse_args(sys.argv[:1] + args)
     if len(args) < 3:
         parser.print_help()
@@ -432,7 +463,7 @@ def do_show_metadata(db, id, as_opf):
     else:
         print unicode(mi).encode(preferred_encoding)
 
-def command_show_metadata(args, dbpath):
+def show_metadata_option_parser():
     parser = get_parser(_(
 '''
 %prog show_metadata [options] id
@@ -442,6 +473,10 @@ id is an id number from the list command.
 '''))
     parser.add_option('--as-opf', default=False, action='store_true',
                       help=_('Print metadata in OPF form (XML)'))
+    return parser
+
+def command_show_metadata(args, dbpath):
+    parser = show_metadata_option_parser()
     opts, args = parser.parse_args(sys.argv[1:]+args)
     if len(args) < 2:
         parser.print_help()
@@ -456,11 +491,10 @@ def do_set_metadata(db, id, stream):
     mi = OPF(stream)
     db.set_metadata(id, mi)
     do_show_metadata(db, id, False)
-    if send_message is not None:
-        send_message('refreshdb:', 'calibre GUI')
+    send_message()
 
-def command_set_metadata(args, dbpath):
-    parser = get_parser(_(
+def set_metadata_option_parser():
+    return get_parser(_(
 '''
 %prog set_metadata [options] id /path/to/metadata.opf
 
@@ -469,6 +503,9 @@ from the OPF file metadata.opf. id is an id number from the list command. You
 can get a quick feel for the OPF format by using the --as-opf switch to the
 show_metadata command.
 '''))
+
+def command_set_metadata(args, dbpath):
+    parser = set_metadata_option_parser()
     opts, args = parser.parse_args(sys.argv[1:]+args)
     if len(args) < 3:
         parser.print_help()
@@ -479,12 +516,20 @@ show_metadata command.
     do_set_metadata(get_db(dbpath, opts), id, opf)
     return 0
 
-def do_export(db, ids, dir, single_dir, by_author):
+def do_export(db, ids, dir, opts):
     if ids is None:
         ids = list(db.all_ids())
-    db.export_to_dir(dir, ids, byauthor=by_author, single_dir=single_dir, index_is_id=True)
+    from calibre.library.save_to_disk import save_to_disk
+    failures = save_to_disk(db, ids, dir, opts=opts)
 
-def command_export(args, dbpath):
+    if failures:
+        prints('Failed to save the following books:')
+        for id, title, tb in failures:
+            prints(str(id)+':', title)
+            prints('\t'+'\n\t'.join(tb.splitlines()))
+            prints(' ')
+
+def export_option_parser():
     parser = get_parser(_('''\
 %prog export [options] ids
 
@@ -498,8 +543,31 @@ an opf file). You can get id numbers from the list command.
                       help=(_('Export books to the specified directory. Default is')+' %default'))
     parser.add_option('--single-dir', default=False, action='store_true',
                       help=_('Export all books into a single directory'))
-    parser.add_option('--by-author', default=False, action='store_true',
-                      help=_('Create file names as author - title instead of title - author'))
+    from calibre.library.save_to_disk import config
+    c = config()
+    for pref in ['asciiize', 'update_metadata', 'write_opf', 'save_cover']:
+        opt = c.get_option(pref)
+        switch = '--dont-'+pref.replace('_', '-')
+        parser.add_option(switch, default=True, action='store_false',
+                help=opt.help+' '+_('Specifying this switch will turn '
+                    'this behavior off.'), dest=pref)
+
+    for pref in ['timefmt', 'template', 'formats']:
+        opt = c.get_option(pref)
+        switch = '--'+pref
+        parser.add_option(switch, default=opt.default,
+                help=opt.help, dest=pref)
+
+    for pref in ('replace_whitespace', 'to_lowercase'):
+        opt = c.get_option(pref)
+        switch = '--'+pref.replace('_', '-')
+        parser.add_option(switch, default=False, action='store_true',
+                help=opt.help)
+
+    return parser
+
+def command_export(args, dbpath):
+    parser = export_option_parser()
     opts, args = parser.parse_args(sys.argv[1:]+args)
     if (len(args) < 2 and not opts.all):
         parser.print_help()
@@ -508,12 +576,14 @@ an opf file). You can get id numbers from the list command.
         return 1
     ids = None if opts.all else map(int, args[1].split(','))
     dir = os.path.abspath(os.path.expanduser(opts.to_dir))
-    do_export(get_db(dbpath, opts), ids, dir, opts.single_dir, opts.by_author)
+    do_export(get_db(dbpath, opts), ids, dir, opts)
     return 0
 
-def main(args=sys.argv):
-    commands = ('list', 'add', 'remove', 'add_format', 'remove_format',
+COMMANDS = ('list', 'add', 'remove', 'add_format', 'remove_format',
                 'show_metadata', 'set_metadata', 'export')
+
+
+def option_parser():
     parser = OptionParser(_(
 '''\
 %%prog command [options] [arguments]
@@ -525,11 +595,16 @@ command is one of:
 
 For help on an individual command: %%prog command --help
 '''
-                          )%'\n  '.join(commands))
+                          )%'\n  '.join(COMMANDS))
+    return parser
+
+
+def main(args=sys.argv):
+    parser = option_parser()
     if len(args) < 2:
         parser.print_help()
         return 1
-    if args[1] not in commands:
+    if args[1] not in COMMANDS:
         if args[1] == '--version':
             parser.print_version()
             return 0
