@@ -9,6 +9,7 @@ from zlib import compress, decompress
 
 from calibre.ebooks.metadata import MetaInformation
 from calibre.web.feeds.recipes import migrate_automatic_profile_to_automatic_recipe
+from calibre.ebooks.metadata import string_to_authors
 
 class Concatenate(object):
     '''String concatenation aggregator for sqlite'''
@@ -27,7 +28,7 @@ class Concatenate(object):
             return self.ans[:-len(self.sep)]
         return self.ans
 class Connection(sqlite.Connection):
-    
+
     def get(self, *args, **kw):
         ans = self.execute(*args)
         if not kw.get('all', True):
@@ -97,7 +98,7 @@ class LibraryDatabase(object):
             obj = conn.execute('INSERT INTO books(title, timestamp, author_sort) VALUES (?,?,?)',
                                (book['title'], book['timestamp'], authors))
             id = obj.lastrowid
-            authors = authors.split('&')
+            authors = string_to_authors(authors)
             for a in authors:
                 author = conn.execute('SELECT id from authors WHERE name=?', (a,)).fetchone()
                 if author:
@@ -785,8 +786,8 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
     FROM books;
 ''')
         conn.execute('pragma user_version=12')
-        conn.commit()    
-        
+        conn.commit()
+
     def __init__(self, dbpath, row_factory=False):
         self.dbpath = dbpath
         self.conn = _connect(dbpath)
@@ -814,8 +815,8 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
 #        _lock_file = None
         self.conn.close()
 
-    @apply
-    def user_version():
+    @dynamic_property
+    def user_version(self):
         doc = 'The user version of this database'
         def fget(self):
             return self.conn.get('pragma user_version;', all=False)
@@ -901,7 +902,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
 
     def id(self, index):
         return self.data[index][0]
-    
+
     def row(self, id):
         for r, record in enumerate(self.data):
             if record[0] == id:
@@ -916,8 +917,8 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
             return _('Unknown')
 
     def authors(self, index, index_is_id=False):
-        ''' 
-        Authors as a comma separated list or None. 
+        '''
+        Authors as a comma separated list or None.
         In the comma separated list, commas in author names are replaced by | symbols
         '''
         if not index_is_id:
@@ -927,6 +928,10 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
         except:
             pass
 
+    def author_id(self, index, index_is_id=False):
+        id  = index if index_is_id else self.id(index)
+        return self.conn.get('SELECT author from books_authors_link WHERE book=?', (id,), all=False)
+
     def isbn(self, idx, index_is_id=False):
         id = idx if index_is_id else self.id(idx)
         return self.conn.get('SELECT isbn FROM books WHERE id=?',(id,), all=False)
@@ -939,11 +944,11 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
         if index_is_id:
             return self.conn.get('SELECT publisher FROM meta WHERE id=?', (index,), all=False)
         return self.data[index][3]
-    
+
     def publisher_id(self, index, index_is_id=False):
         id  = index if index_is_id else self.id(index)
         return self.conn.get('SELECT publisher from books_publishers_link WHERE book=?', (id,), all=False)
-    
+
     def rating(self, index, index_is_id=False):
         if index_is_id:
             return self.conn.get('SELECT rating FROM meta WHERE id=?', (index,), all=False)
@@ -983,7 +988,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
     def series(self, index, index_is_id=False):
         id = self.series_id(index, index_is_id)
         return self.conn.get('SELECT name from series WHERE id=?', (id,), all=False)
-        
+
     def series_index(self, index, index_is_id=False):
         ans = None
         if not index_is_id:
@@ -991,9 +996,9 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
         else:
             ans = self.conn.get('SELECT series_index FROM books WHERE id=?', (index,), all=False)
         try:
-            return int(ans)
+            return float(ans)
         except:
-            return 1
+            return 1.0
 
     def books_in_series(self, series_id):
         '''
@@ -1010,7 +1015,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
 
     def books_in_series_of(self, index, index_is_id=False):
         '''
-        Return an ordered list of all books in the series that the book indetified by index belongs to.
+        Return an ordered list of all books in the series that the book identified by index belongs to.
         If the book does not belong to a series return an empty list. The list contains book ids.
         '''
         series_id = self.series_id(index, index_is_id=index_is_id)
@@ -1021,7 +1026,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
         '''Comments as string or None'''
         id = index if index_is_id else self.id(index)
         return self.conn.get('SELECT text FROM comments WHERE book=?', (id,), all=False)
-        
+
     def formats(self, index, index_is_id=False):
         ''' Return available formats as a comma separated list '''
         id = index if index_is_id else self.id(index)
@@ -1041,11 +1046,23 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
     def all_series(self):
         return [ (i[0], i[1]) for i in \
                 self.conn.get('SELECT id, name FROM series')]
-        
+
+    def series_name(self, series_id):
+        return self.conn.get('SELECT name FROM series WHERE id=%d'%series_id,
+                all=False)
+
+    def author_name(self, author_id):
+        return self.conn.get('SELECT name FROM authors WHERE id=%d'%author_id,
+                all=False)
+
+    def tag_name(self, tag_id):
+        return self.conn.get('SELECT name FROM tags WHERE id=%d'%tag_id,
+                all=False)
+
     def all_authors(self):
         return [ (i[0], i[1]) for i in \
                 self.conn.get('SELECT id, name FROM authors')]
-    
+
     def all_publishers(self):
         return [ (i[0], i[1]) for i in \
                 self.conn.get('SELECT id, name FROM publishers')]
@@ -1053,12 +1070,21 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
     def all_tags(self):
         return [i[0].strip() for i in self.conn.get('SELECT name FROM tags') if i[0].strip()]
 
+    def all_tags2(self):
+        return [ (i[0], i[1]) for i in \
+                self.conn.get('SELECT id, name FROM tags')]
+
+
     def conversion_options(self, id, format):
         data = self.conn.get('SELECT data FROM conversion_options WHERE book=? AND format=?', (id, format.upper()), all=False)
         if data:
             return cPickle.loads(str(data))
         return None
 
+    def delete_conversion_options(self, id, format):
+        self.conn.execute('DELETE FROM conversion_options WHERE book=? AND format=?',
+                (id, format.upper()))
+
 
     def add_format(self, index, ext, stream, index_is_id=False):
         '''
@@ -1103,7 +1129,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
                 item[col] = val
                 break
         if column == 'authors':
-            val = val.split('&,')
+            val = string_to_authors(val)
             self.set_authors(id, val)
         elif column == 'title':
             self.set_title(id, val)
@@ -1266,7 +1292,7 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
                 mi.authors = ['Unknown']
         authors = []
         for a in mi.authors:
-            authors += a.split('&')
+            authors += string_to_authors(a)
         self.set_authors(id, authors)
         if mi.author_sort:
             self.set_author_sort(id, mi.author_sort)
@@ -1278,9 +1304,9 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
             self.set_series(id, mi.series)
         if mi.cover_data[1] is not None:
             self.set_cover(id, mi.cover_data[1])
-    
-        
-            
+
+
+
 
     def add_books(self, paths, formats, metadata, uris=[], add_duplicates=True):
         '''
@@ -1385,16 +1411,16 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
     def all_ids(self):
         return [i[0] for i in self.conn.get('SELECT id FROM books')]
 
-                         
 
 
-    
+
+
 
     def has_id(self, id):
         return self.conn.get('SELECT id FROM books where id=?', (id,), all=False) is not None
 
-    
-    
+
+
 
 class SearchToken(object):
 
