@@ -6,14 +6,14 @@ __copyright__ = '2008, Darko Miletic <darko.miletic at gmail.com>'
 engadget.com
 '''
 
-import string,re
 from calibre.web.feeds.news import BasicNewsRecipe
 
 class Engadget(BasicNewsRecipe):
     title                 = u'Engadget'
     __author__            = 'Darko Miletic'
     description           = 'Tech news'
-    language = _('English')    
+    language = 'en'
+
     oldest_article        = 7
     max_articles_per_feed = 100
     no_stylesheets        = True
