@@ -10,7 +10,8 @@ class Freakonomics(BasicNewsRecipe):
     title = 'Freakonomics Blog'
     description = 'The Hidden side of everything'
     __author__ = 'Kovid Goyal'
-    language = _('English')
+    language = 'en'
+
     
     feeds = [('Blog', 'http://freakonomics.blogs.nytimes.com/feed/atom/')]
     
