@@ -6,14 +6,14 @@ __copyright__ = '2008, Darko Miletic <darko.miletic at gmail.com>'
 fudzilla.com
 '''
 
-import string,re
 from calibre.web.feeds.news import BasicNewsRecipe
 
 class Fudzilla(BasicNewsRecipe):
     title                 = u'Fudzilla'
     __author__            = 'Darko Miletic'
-    language = _('English')
-    description           = 'Tech news'    
+    language = 'en'
+
+    description           = 'Tech news'
     oldest_article        = 7
     max_articles_per_feed = 100
     no_stylesheets        = True
