@@ -5,7 +5,8 @@ class PhysicsWorld(BasicNewsRecipe):
     title          = u'Physicsworld'
     description    = 'News from the world of physics'
     __author__     = 'Hypernova'
-    language       = _('English')
+    language = 'en'
+
     oldest_article = 7
     max_articles_per_feed = 100
     no_stylesheets        = True
@@ -17,11 +18,11 @@ class PhysicsWorld(BasicNewsRecipe):
     preprocess_regexps = [
    (re.compile(r'<div id="shareThis">.*</body>', re.DOTALL|re.IGNORECASE),
     lambda match: '</body>'),
-]    
+]
     feeds          = [
                           (u'Headlines News', u'http://feeds.feedburner.com/PhysicsWorldNews')
                       ]
-    
+
     def get_browser(self):
         br = BasicNewsRecipe.get_browser(self)
         if self.username is not None and self.password is not None:
@@ -31,5 +32,5 @@ class PhysicsWorld(BasicNewsRecipe):
             br['password'] = self.password
             br.submit()
         return br
-        
-        
\ No newline at end of file
+
+
