@@ -12,7 +12,8 @@ class Sciencenews(BasicNewsRecipe):
     __author__            = u'Darko Miletic'
     description           = u"Science News is an award-winning weekly newsmagazine covering the most important research in all fields of science. Its 16 pages each week are packed with short, accurate articles that appeal to both general readers and scientists. Published since 1922, the magazine now reaches about 150,000 subscribers and more than 1 million readers. These are the latest News Items from Science News."
     oldest_article        = 30
-    language = _('English')
+    language = 'en'
+
     max_articles_per_feed = 100
     no_stylesheets        = True
     use_embedded_content  = False
