@@ -7,18 +7,19 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 Fetch a webpage and its links recursively. The webpages are saved to disk in
 UTF-8 encoding with any charset declarations removed.
 '''
-import sys, socket, os, urlparse, logging, re, time, copy, urllib2, threading, traceback
+import sys, socket, os, urlparse, re, time, copy, urllib2, threading, traceback
 from urllib import url2pathname, quote
 from threading import RLock
 from httplib import responses
 from PIL import Image
 from cStringIO import StringIO
 
-from calibre import setup_cli_handlers, browser, sanitize_file_name, \
-                    relpath, LoggingInterface, unicode_path
+from calibre import browser, relpath, unicode_path
+from calibre.utils.filenames import ascii_filename
 from calibre.ebooks.BeautifulSoup import BeautifulSoup, Tag
 from calibre.ebooks.chardet import xml_to_unicode
 from calibre.utils.config import OptionParser
+from calibre.utils.logging import Log
 
 class FetchError(Exception):
     pass
@@ -28,10 +29,10 @@ class closing(object):
 
     def __init__(self, thing):
         self.thing = thing
-    
+
     def __enter__(self):
         return self.thing
-    
+
     def __exit__(self, *exc_info):
         try:
             self.thing.close()
@@ -52,54 +53,60 @@ def save_soup(soup, target):
     ns = BeautifulSoup('<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />')
     nm = ns.find('meta')
     metas = soup.findAll('meta', content=True)
+    added = False
     for meta in metas:
         if 'charset' in meta.get('content', '').lower():
             meta.replaceWith(nm)
-    
+            added = True
+    if not added:
+        head = soup.find('head')
+        if head is not None:
+            head.insert(0, nm)
+
     selfdir = os.path.dirname(target)
-    
+
     for tag in soup.findAll(['img', 'link', 'a']):
         for key in ('src', 'href'):
             path = tag.get(key, None)
             if path and os.path.isfile(path) and os.path.exists(path) and os.path.isabs(path):
                 tag[key] = unicode_path(relpath(path, selfdir).replace(os.sep, '/'))
-    
+
     html = unicode(soup)
     with open(target, 'wb') as f:
+        idx = html.find('hoping')
         f.write(html.encode('utf-8'))
-    
+
 class response(str):
-    
+
     def __new__(cls, *args):
         obj = super(response, cls).__new__(cls, *args)
         obj.newurl = None
         return obj
-        
+
 class DummyLock(object):
-    
+
     def __enter__(self, *args): return self
     def __exit__(self, *args): pass
 
-class RecursiveFetcher(object, LoggingInterface):
-    LINK_FILTER = tuple(re.compile(i, re.IGNORECASE) for i in 
+class RecursiveFetcher(object):
+    LINK_FILTER = tuple(re.compile(i, re.IGNORECASE) for i in
                 ('.exe\s*$', '.mp3\s*$', '.ogg\s*$', '^\s*mailto:', '^\s*$'))
     #ADBLOCK_FILTER = tuple(re.compile(i, re.IGNORECASE) for it in
     #                       (
-    #                        
+    #
     #                        )
     #                       )
     CSS_IMPORT_PATTERN = re.compile(r'\@import\s+url\((.*?)\)', re.IGNORECASE)
     default_timeout = socket.getdefaulttimeout() # Needed here as it is used in __del__
     DUMMY_LOCK = DummyLock()
-    
-    def __init__(self, options, logger, image_map={}, css_map={}, job_info=None):
-        LoggingInterface.__init__(self, logger)
+
+    def __init__(self, options, log, image_map={}, css_map={}, job_info=None):
         self.base_dir = os.path.abspath(os.path.expanduser(options.dir))
         if not os.path.exists(self.base_dir):
             os.makedirs(self.base_dir)
-        self.default_timeout = socket.getdefaulttimeout()
-        socket.setdefaulttimeout(options.timeout)
+        self.log = log
         self.verbose = options.verbose
+        self.timeout = options.timeout
         self.encoding = options.encoding
         self.browser = options.browser if hasattr(options, 'browser') else browser()
         self.max_recursions = options.max_recursions
@@ -123,19 +130,19 @@ class RecursiveFetcher(object, LoggingInterface):
         self.remove_tags_after   = getattr(options, 'remove_tags_after', None)
         self.remove_tags_before  = getattr(options, 'remove_tags_before', None)
         self.keep_only_tags      = getattr(options, 'keep_only_tags', [])
-        self.preprocess_html_ext = getattr(options, 'preprocess_html', lambda soup: soup) 
+        self.preprocess_html_ext = getattr(options, 'preprocess_html', lambda soup: soup)
         self.postprocess_html_ext= getattr(options, 'postprocess_html', None)
         self.download_stylesheets = not options.no_stylesheets
         self.show_progress = True
         self.failed_links = []
         self.job_info = job_info
-        
+
     def get_soup(self, src):
         nmassage = copy.copy(BeautifulSoup.MARKUP_MASSAGE)
         nmassage.extend(self.preprocess_regexps)
         nmassage += [(re.compile(r'<!DOCTYPE .+?>', re.DOTALL), lambda m: '')] # Some websites have buggy doctype declarations that mess up beautifulsoup
         soup = BeautifulSoup(xml_to_unicode(src, self.verbose, strip_encoding_pats=True)[0], markupMassage=nmassage)
-         
+
         if self.keep_only_tags:
             body = Tag(soup, 'body')
             try:
@@ -147,7 +154,7 @@ class RecursiveFetcher(object, LoggingInterface):
                 soup.find('body').replaceWith(body)
             except AttributeError: # soup has no body element
                 pass
-            
+
         def remove_beyond(tag, next):
             while tag is not None and tag.name != 'body':
                 after = getattr(tag, next)
@@ -156,34 +163,37 @@ class RecursiveFetcher(object, LoggingInterface):
                     after.extract()
                     after = ns
                 tag = tag.parent
-        
+
         if self.remove_tags_after is not None:
             rt = [self.remove_tags_after] if isinstance(self.remove_tags_after, dict) else self.remove_tags_after
             for spec in rt:
                 tag = soup.find(**spec)
                 remove_beyond(tag, 'nextSibling')
-            
+
         if self.remove_tags_before is not None:
             tag = soup.find(**self.remove_tags_before)
             remove_beyond(tag, 'previousSibling')
-            
+
         for kwds in self.remove_tags:
             for tag in soup.findAll(**kwds):
                 tag.extract()
         return self.preprocess_html_ext(soup)
-        
-    
+
+
     def fetch_url(self, url):
         data = None
-        self.log_debug('Fetching %s', url)
-        delta = time.time() - self.last_fetch_at 
+        self.log.debug('Fetching', url)
+        delta = time.time() - self.last_fetch_at
         if  delta < self.delay:
             time.sleep(delta)
-        if re.search(r'\s+', url) is not None:
-            url = quote(url)
+        if re.search(r'\s+|,', url) is not None:
+            purl = list(urlparse.urlparse(url))
+            for i in range(2, 6):
+                purl[i] = quote(purl[i])
+            url = urlparse.urlunparse(purl)
         with self.browser_lock:
             try:
-                with closing(self.browser.open(url)) as f:
+                with closing(self.browser.open(url, timeout=self.timeout)) as f:
                     data = response(f.read()+f.read())
                     data.newurl = f.geturl()
             except urllib2.URLError, err:
@@ -191,43 +201,43 @@ class RecursiveFetcher(object, LoggingInterface):
                     raise FetchError, responses[err.code]
                 if getattr(err, 'reason', [0])[0] == 104 or \
                     getattr(getattr(err, 'args', [None])[0], 'errno', None) == -2: # Connection reset by peer or Name or service not know
-                    self.log_debug('Temporary error, retrying in 1 second')
+                    self.log.debug('Temporary error, retrying in 1 second')
                     time.sleep(1)
-                    with closing(self.browser.open(url)) as f:
+                    with closing(self.browser.open(url, timeout=self.timeout)) as f:
                         data = response(f.read()+f.read())
                         data.newurl = f.geturl()
-                else: 
+                else:
                     raise err
             finally:
                 self.last_fetch_at = time.time()
             return data
 
-        
+
     def start_fetch(self, url):
         soup = BeautifulSoup(u'<a href="'+url+'" />')
-        self.log_info('Downloading')
+        self.log.debug('Downloading')
         res = self.process_links(soup, url, 0, into_dir='')
-        self.log_info('%s saved to %s', url, res)
+        self.log.debug('%s saved to %s'%( url, res))
         return res
-    
+
     def is_link_ok(self, url):
         for i in self.__class__.LINK_FILTER:
             if i.search(url):
                 return False
         return True
-        
+
     def is_link_wanted(self, url):
         if self.filter_regexps:
             for f in self.filter_regexps:
                 if f.search(url):
-                    return False            
+                    return False
         if self.match_regexps:
             for m in self.match_regexps:
                 if m.search(url):
                     return True
             return False
         return True
-        
+
     def process_stylesheets(self, soup, baseurl):
         diskpath = unicode_path(os.path.join(self.current_dir, 'stylesheets'))
         if not os.path.exists(diskpath):
@@ -244,8 +254,7 @@ class RecursiveFetcher(object, LoggingInterface):
                 try:
                     data = self.fetch_url(iurl)
                 except Exception, err:
-                    self.log_debug('Could not fetch stylesheet %s', iurl)
-                    self.log_debug('Error: %s', str(err), exc_info=True)
+                    self.log.exception('Could not fetch stylesheet %s'% iurl)
                     continue
                 stylepath = os.path.join(diskpath, 'style'+str(c)+'.css')
                 with self.stylemap_lock:
@@ -254,7 +263,7 @@ class RecursiveFetcher(object, LoggingInterface):
                     x.write(data)
                 tag['href'] = stylepath
             else:
-                for ns in tag.findAll(text=True):                    
+                for ns in tag.findAll(text=True):
                     src = str(ns)
                     m = self.__class__.CSS_IMPORT_PATTERN.search(src)
                     if m:
@@ -268,8 +277,7 @@ class RecursiveFetcher(object, LoggingInterface):
                         try:
                             data = self.fetch_url(iurl)
                         except Exception, err:
-                            self.log_warning('Could not fetch stylesheet %s', iurl)
-                            self.log_debug('Error: %s', str(err), exc_info=True)
+                            self.log.exception('Could not fetch stylesheet %s'% iurl)
                             continue
                         c += 1
                         stylepath = os.path.join(diskpath, 'style'+str(c)+'.css')
@@ -278,9 +286,9 @@ class RecursiveFetcher(object, LoggingInterface):
                         with open(stylepath, 'wb') as x:
                             x.write(data)
                         ns.replaceWith(src.replace(m.group(1), stylepath))
-                        
-                        
-    
+
+
+
     def process_images(self, soup, baseurl):
         diskpath = unicode_path(os.path.join(self.current_dir, 'images'))
         if not os.path.exists(diskpath):
@@ -292,9 +300,6 @@ class RecursiveFetcher(object, LoggingInterface):
                 iurl = self.image_url_processor(baseurl, iurl)
             ext  = os.path.splitext(iurl)[1]
             ext  = ext[:5]
-            #if not ext:
-            #    self.log_debug('Skipping extensionless image %s', iurl)
-            #    continue
             if not urlparse.urlsplit(iurl).scheme:
                 iurl = urlparse.urljoin(baseurl, iurl, False)
             with self.imagemap_lock:
@@ -304,11 +309,10 @@ class RecursiveFetcher(object, LoggingInterface):
             try:
                 data = self.fetch_url(iurl)
             except Exception, err:
-                self.log_warning('Could not fetch image %s', iurl)
-                self.log_debug('Error: %s', str(err), exc_info=True)
+                self.log.exception('Could not fetch image %s'% iurl)
                 continue
             c += 1
-            fname = sanitize_file_name('img'+str(c)+ext)
+            fname = ascii_filename('img'+str(c)+ext)
             if isinstance(fname, unicode):
                 fname = fname.encode('ascii', 'replace')
             imgpath = os.path.join(diskpath, fname+'.jpg')
@@ -323,7 +327,7 @@ class RecursiveFetcher(object, LoggingInterface):
                 traceback.print_exc()
                 continue
 
-    def absurl(self, baseurl, tag, key, filter=True): 
+    def absurl(self, baseurl, tag, key, filter=True):
         iurl = tag[key]
         parts = urlparse.urlsplit(iurl)
         if not parts.netloc and not parts.path:
@@ -331,32 +335,32 @@ class RecursiveFetcher(object, LoggingInterface):
         if not parts.scheme:
             iurl = urlparse.urljoin(baseurl, iurl, False)
         if not self.is_link_ok(iurl):
-            self.log_debug('Skipping invalid link: %s', iurl)
+            self.log.debug('Skipping invalid link:', iurl)
             return None
         if filter and not self.is_link_wanted(iurl):
-            self.log_debug('Filtered link: '+iurl)
+            self.log.debug('Filtered link: '+iurl)
             return None
         return iurl
-    
+
     def normurl(self, url):
         parts = list(urlparse.urlsplit(url))
         parts[4] = ''
         return urlparse.urlunsplit(parts)
-                
+
     def localize_link(self, tag, key, path):
         parts = urlparse.urlsplit(tag[key])
         suffix = '#'+parts.fragment if parts.fragment else ''
         tag[key] = path+suffix
-    
+
     def process_return_links(self, soup, baseurl):
         for tag in soup.findAll(lambda tag: tag.name.lower()=='a' and tag.has_key('href')):
-            iurl = self.absurl(baseurl, tag, 'href')            
+            iurl = self.absurl(baseurl, tag, 'href')
             if not iurl:
                 continue
             nurl = self.normurl(iurl)
             if self.filemap.has_key(nurl):
                 self.localize_link(tag, 'href', self.filemap[nurl])
-    
+
     def process_links(self, soup, baseurl, recursion_level, into_dir='links'):
         res = ''
         diskpath = os.path.join(self.current_dir, into_dir)
@@ -366,7 +370,7 @@ class RecursiveFetcher(object, LoggingInterface):
         try:
             self.current_dir = diskpath
             tags = list(soup.findAll('a', href=True))
-            
+
             for c, tag in enumerate(tags):
                 if self.show_progress:
                     print '.',
@@ -393,80 +397,74 @@ class RecursiveFetcher(object, LoggingInterface):
                        len(re.compile('<!--.*?-->', re.DOTALL).sub('', dsrc).strip()) == 0:
                         raise ValueError('No content at URL %s'%iurl)
                     if self.encoding is not None:
-                        dsrc = dsrc.decode(self.encoding, 'ignore')
+                        dsrc = dsrc.decode(self.encoding, 'replace')
                     else:
                         dsrc = xml_to_unicode(dsrc, self.verbose)[0]
-                    
+
                     soup = self.get_soup(dsrc)
-                    
+
                     base = soup.find('base', href=True)
                     if base is not None:
                         newbaseurl = base['href']
-                    self.log_debug('Processing images...')
+                    self.log.debug('Processing images...')
                     self.process_images(soup, newbaseurl)
                     if self.download_stylesheets:
                         self.process_stylesheets(soup, newbaseurl)
-                    
+
                     _fname = basename(iurl)
                     if not isinstance(_fname, unicode):
                         _fname.decode('latin1', 'replace')
                     _fname = _fname.encode('ascii', 'replace').replace('%', '').replace(os.sep, '')
-                    _fname = sanitize_file_name(_fname)
+                    _fname = ascii_filename(_fname)
                     _fname = os.path.splitext(_fname)[0]+'.xhtml'
                     res = os.path.join(linkdiskpath, _fname)
                     self.downloaded_paths.append(res)
                     self.filemap[nurl] = res
                     if recursion_level < self.max_recursions:
-                        self.log_debug('Processing links...')
+                        self.log.debug('Processing links...')
                         self.process_links(soup, newbaseurl, recursion_level+1)
                     else:
-                        self.process_return_links(soup, newbaseurl) 
-                        self.log_debug('Recursion limit reached. Skipping links in %s', iurl)
-                    
+                        self.process_return_links(soup, newbaseurl)
+                        self.log.debug('Recursion limit reached. Skipping links in', iurl)
+
                     if callable(self.postprocess_html_ext):
-                        soup = self.postprocess_html_ext(soup, 
+                        soup = self.postprocess_html_ext(soup,
                                 c==0 and recursion_level==0 and not getattr(self, 'called_first', False),
                                 self.job_info)
-                        
+
                         if c==0 and recursion_level == 0:
                             self.called_first = True
-                    
+
                     save_soup(soup, res)
                     self.localize_link(tag, 'href', res)
                 except Exception, err:
                     self.failed_links.append((iurl, traceback.format_exc()))
-                    self.log_warning('Could not fetch link %s', iurl)
-                    self.log_debug('Error: %s', str(err), exc_info=True)
+                    self.log.exception('Could not fetch link', iurl)
                 finally:
                     self.current_dir = diskpath
-                    self.files += 1                
+                    self.files += 1
         finally:
             self.current_dir = prev_dir
         if self.show_progress:
             print
         return res
-    
-    def __del__(self):
-        dt = getattr(self, 'default_timeout', None)
-        if dt is not None:
-            socket.setdefaulttimeout(dt)
-        
+
 def option_parser(usage=_('%prog URL\n\nWhere URL is for example http://google.com')):
     parser = OptionParser(usage=usage)
-    parser.add_option('-d', '--base-dir', 
+    parser.add_option('-d', '--base-dir',
                       help=_('Base directory into which URL is saved. Default is %default'),
                       default='.', type='string', dest='dir')
-    parser.add_option('-t', '--timeout', 
+    parser.add_option('-t', '--timeout',
                       help=_('Timeout in seconds to wait for a response from the server. Default: %default s'),
                       default=10.0, type='float', dest='timeout')
-    parser.add_option('-r', '--max-recursions', default=1, 
+    parser.add_option('-r', '--max-recursions', default=1,
                       help=_('Maximum number of levels to recurse i.e. depth of links to follow. Default %default'),
                       type='int', dest='max_recursions')
     parser.add_option('-n', '--max-files', default=sys.maxint, type='int', dest='max_files',
                       help=_('The maximum number of files to download. This only applies to files from <a href> tags. Default is %default'))
     parser.add_option('--delay', default=0, dest='delay', type='int',
                       help=_('Minimum interval in seconds between consecutive fetches. Default is %default s'))
-    parser.add_option('--encoding', default=None, 
+    parser.add_option('--encoding', default=None,
                       help=_('The character encoding for the websites you are trying to download. The default is to try and guess the encoding.'))
     parser.add_option('--match-regexp', default=[], action='append', dest='match_regexps',
                       help=_('Only links that match this regular expression will be followed. This option can be specified multiple times, in which case as long as a link matches any one regexp, it will be followed. By default all links are followed.'))
@@ -479,23 +477,21 @@ def option_parser(usage=_('%prog URL\n\nWhere URL is for example http://google.c
     return parser
 
 
-def create_fetcher(options, logger=None, image_map={}):
-    if logger is None:
-        level = logging.DEBUG if options.verbose else logging.INFO
-        logger = logging.getLogger('web2disk')
-        setup_cli_handlers(logger, level)
-    return RecursiveFetcher(options, logger, image_map={})
+def create_fetcher(options, image_map={}, log=None):
+    if log is None:
+        log = Log()
+    return RecursiveFetcher(options, log, image_map={})
 
 def main(args=sys.argv):
-    parser = option_parser()    
+    parser = option_parser()
     options, args = parser.parse_args(args)
     if len(args) != 2:
         parser.print_help()
         return 1
-    
-    fetcher = create_fetcher(options) 
-    fetcher.start_fetch(args[1])
-    
 
-if __name__ == '__main__':    
+    fetcher = create_fetcher(options)
+    fetcher.start_fetch(args[1])
+
+
+if __name__ == '__main__':
     sys.exit(main())
